@@ -1,4 +1,4 @@
-/* Copyright 2004,2007-2013 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2004,2007-2013,2015,2016 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -63,8 +63,8 @@
 /**                                 to     09 jan 2004     **/
 /**                # Version 5.1  : from : 11 dec 2007     **/
 /**                                 to     25 jun 2010     **/
-/**                # Version 6.0  : from : 14 fev 2011     **/
-/**                                 to     26 nov 2013     **/
+/**                # Version 6.0  : from : 14 feb 2011     **/
+/**                                 to     22 feb 2016     **/
 /**                                                        **/
 /************************************************************/
 
@@ -76,13 +76,16 @@
 
 #include "module.h"
 #include "common.h"
+#include "graph.h"
 #include "arch.h"
 #include "arch_cmplt.h"
 #include "arch_cmpltw.h"
 #include "arch_deco.h"
+#include "arch_deco2.h"
 #include "arch_dist.h"
 #include "arch_hcub.h"
 #include "arch_mesh.h"
+#include "arch_sub.h"
 #include "arch_tleaf.h"
 #include "arch_torus.h"
 #include "arch_vcmplt.h"
@@ -95,6 +98,7 @@
 static const ArchClass      archClassTab[] = { ARCHCLASSBLOCK ("cmplt",    Cmplt,  ARCHPART),
                                                ARCHCLASSBLOCK ("cmpltw",   Cmpltw, ARCHPART),
                                                ARCHCLASSBLOCK ("deco",     Deco,   ARCHNONE),
+                                               ARCHCLASSBLOCK ("deco",     Deco2,  ARCHNONE), /* Hidden, type-2 decomposition-defined architecture */
                                                ARCHCLASSBLOCK ("dist",     Dist,   ARCHNONE),
                                                ARCHCLASSBLOCK ("hcub",     Hcub,   ARCHNONE),
                                                ARCHCLASSBLOCK ("tleaf",    Tleaf,  ARCHNONE),
@@ -105,6 +109,8 @@ static const ArchClass      archClassTab[] = { ARCHCLASSBLOCK ("cmplt",    Cmplt
                                                ARCHCLASSBLOCK ("mesh2U",   Mesh2u, ARCHNONE),
 #endif /* SCOTCH_DEBUG_ARCH3 */
                                                ARCHCLASSBLOCK ("mesh3D",   Mesh3,  ARCHNONE),
+                                               ARCHCLASSBLOCK ("meshXD",   MeshX,  ARCHNONE),
+                                               ARCHCLASSBLOCK ("sub",      Sub,    ARCHNONE),
                                                ARCHCLASSBLOCK ("torus2D",  Torus2, ARCHNONE),
                                                ARCHCLASSBLOCK ("torus3D",  Torus3, ARCHNONE),
                                                ARCHCLASSBLOCK ("torusXD",  TorusX, ARCHNONE),
@@ -164,9 +170,12 @@ Arch * restrict const       archptr)
 
   o = 0;                                          /* Assume everything will be all right */
   if ((archptr->class           != NULL) &&
-      (archptr->class->archFree != NULL))         /* If there is a specific freeing routing                */
+      (archptr->class->archFree != NULL))         /* If there is a specific freeing routine                */
     o = archptr->class->archFree (&archptr->data); /* Call it                                              */
-  memSet (archptr, 0, sizeof (Arch));             /* Initialize the architecture body (arch->class = NULL) */
+
+#ifdef SCOTCH_DEBUG_GRAPH2
+  memSet (archptr, ~0, sizeof (Arch));            /* Purge graph fields */
+#endif /* SCOTCH_DEBUG_GRAPH2 */
 
   return (o);
 }
@@ -196,6 +205,9 @@ FILE * const                stream)
     return     (1);
   }
 
+  archptr->class   = class;                       /* Set architecture class                                                       */
+  archptr->flagval = archptr->class->flagval | ARCHFREE; /* Copy architecture flag before it can be modified and set freeing flag */
+
   if (class->archLoad != NULL) {                  /* If class has loading function */
     if (class->archLoad (&archptr->data, stream) != 0) { /* Load class data        */
       errorPrint ("archLoad: cannot load architecture data");
@@ -204,8 +216,6 @@ FILE * const                stream)
       return (1);
     }
   }
-  archptr->class   = class;                       /* Set architecture class */
-  archptr->flagval = archptr->class->flagval;     /* Copy architecture flag */
 
   return (0);
 }
@@ -230,7 +240,6 @@ FILE * restrict const       stream)
                 archptr->class->archname) == EOF);
   if (archptr->class->archSave != NULL)           /* If class has saving function      */
     o |= archptr->class->archSave (&archptr->data, stream); /* Write architecture data */
-  o |= (fprintf (stream, "\n") == EOF);
   if (o != 0)
     errorPrint ("archSave: bad output");
 
@@ -248,15 +257,23 @@ FILE * restrict const       stream)
 const ArchClass *
 archClass (
 const char * const          name)
+{
+  return (archClass2 (name, 0));                  /* Get first instance of class name */
+}
+
+const ArchClass *
+archClass2 (
+const char * const          name,
+const int                   num)
 {
   const ArchClass * restrict  class;              /* Pointer to architecture class */
 
-  for (class = archClassTab;                      /* For all classes             */
-       (class->archname != NULL) &&               /* Search if class names match */
-       (strcasecmp (name, class->archname) != 0);
-       class ++) ;
+  for (class = archClassTab; class->archname != NULL; class ++) { /* For all classes */
+    if (strcasecmp (name, class->archname) == 0)  /* If class names matches          */
+      return (class + num);                       /* Return proper class             */
+  }
 
-  return ((class->archname != NULL) ? class : NULL);
+  return (NULL);                                  /* Class not found */
 }
 
 /**************************************/
