@@ -1,4 +1,4 @@
-/* Copyright 2004,2007,2008,2010,2011 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007,2008,2010,2011,2014,2016,2018 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -34,7 +34,7 @@
 /**   NAME       : arch_build.c                            **/
 /**                                                        **/
 /**   AUTHOR     : Francois PELLEGRINI                     **/
-/**                Sebastien FOURESTIER                    **/
+/**                Sebastien FOURESTIER (v6.0)             **/
 /**                                                        **/
 /**   FUNCTION   : This module builds a decomposition-     **/
 /**                based architecture from a source graph. **/
@@ -51,6 +51,8 @@
 /**                                 to     03 apr 2008     **/
 /**                # Version 5.1  : from : 28 sep 2008     **/
 /**                                 to     28 jun 2011     **/
+/**                # Version 6.0  : from : 28 jun 2011     **/
+/**                                 to     15 may 2018     **/
 /**                                                        **/
 /************************************************************/
 
@@ -86,10 +88,10 @@
 
 static
 void
-archBuildJobExit (
-ArchBuildJob * const        jobtab)
+archDecoBuildJobExit (
+ArchDecoBuildJob * const    jobtab)
 {
-  ArchBuildJob *          jobptr;
+  ArchDecoBuildJob *      jobptr;
 
   jobptr = jobtab;
   do {
@@ -115,35 +117,35 @@ ArchBuildJob * const        jobtab)
 */
 
 int
-archBuild (
+archDecoArchBuild (
 Arch * restrict const       tgtarchptr,           /*+ Decomposition architecture to build    +*/
 const Graph * const         tgtgrafptr,           /*+ Source graph modeling the architecture +*/
 const VertList * const      tgtlistptr,           /*+ Subset of source graph vertices        +*/
 const Strat * const         mapstrat)             /*+ Bipartitioning strategy                +*/
 {
-  Gnum * restrict               mapparttax;       /* Based access to mapping part array       */
-  ArchDom                       domsub0;          /* Temporary space for subdomain 0          */
-  Gnum                          termdomnbr;       /* Number of terminal domains               */
-  Gnum                          termdommax;       /* Maximum terminal number                  */
-  ArchDecoTermVert * restrict   termverttab;      /* Terminal vertex table                    */
-  Anum * restrict               termdisttab;      /* Vertex distance table                    */
-  ArchBuildDistElem * restrict  disttax;          /* Distance table                           */
-  ArchBuildQueuElem * restrict  queutab;          /* Distance queue table                     */
-  Gnum                          queuhead;         /* Head-of-queue index                      */
-  Gnum                          queutail;         /* Tail-of-queue index                      */
-  Mapping                       mappdat;          /* Partial and final mapping data           */
-  ArchBuildJob * restrict       jobtab;           /* Job array                                */
-  ArchBuildJob *                joblink;          /* Linked list of jobs to process           */
-  ArchBuildJob *                joborgptr;        /* Pointer to original job and first subjob */
-  ArchBuildJob *                jobsubptr;        /* Pointer to second subjob                 */
-  Bgraph                        actgrafdat;       /* Active graph for bipartitioning          */
-  Gnum                          invedlosiz;       /* Size of inversed edge load array         */
-  Gnum * restrict               invedlotax;       /* Inversed edge load array for cutting     */
-  Gnum * restrict               actfrontab;       /* Frontier array for all bipartitionings   */
-  GraphPart * restrict          actparttax;       /* Part array for all bipartitionings       */
-  GraphPart                     actpartval;       /* Part value to put to subjob              */
-  Gnum                          actpartnbr;       /* Size of part value to put to subjob      */
-  Gnum                          termdomnum;
+  Arch                              archdat;      /* Variable-sized architecture for bipartitioning */
+  ArchDom                           domsub0;      /* Temporary space for subdomain 0                */
+  Gnum                              termdomnbr;   /* Number of terminal domains                     */
+  Gnum                              termdommax;   /* Maximum terminal number                        */
+  ArchDecoTermVert * restrict       termverttab;  /* Terminal vertex table                          */
+  Anum * restrict                   termdisttab;  /* Vertex distance table                          */
+  ArchDecoBuildDistElem * restrict  disttax;      /* Distance table                                 */
+  ArchDecoBuildQueuElem * restrict  queutab;      /* Distance queue table                           */
+  Gnum                              queuhead;     /* Head-of-queue index                            */
+  Gnum                              queutail;     /* Tail-of-queue index                            */
+  Mapping                           mappdat;      /* Partial and final mapping data                 */
+  ArchDecoBuildJob * restrict       jobtab;       /* Job array                                      */
+  ArchDecoBuildJob *                joblink;      /* Linked list of jobs to process                 */
+  ArchDecoBuildJob *                joborgptr;    /* Pointer to original job and first subjob       */
+  ArchDecoBuildJob *                jobsubptr;    /* Pointer to second subjob                       */
+  Bgraph                            actgrafdat;   /* Active graph for bipartitioning                */
+  Gnum                              invedlosiz;   /* Size of inversed edge load array               */
+  Gnum * restrict                   invedlotax;   /* Inversed edge load array for cutting           */
+  Gnum * restrict                   actfrontab;   /* Frontier array for all bipartitionings         */
+  GraphPart * restrict              actparttax;   /* Part array for all bipartitionings             */
+  GraphPart                         actpartval;   /* Part value to put to subjob                    */
+  Gnum                              actpartnbr;   /* Size of part value to put to subjob            */
+  Gnum                              termdomnum;
 
   const Gnum * restrict const tgtverttax = tgtgrafptr->verttax;
   const Gnum * restrict const tgtvendtax = tgtgrafptr->vendtax;
@@ -161,13 +163,13 @@ const Strat * const         mapstrat)             /*+ Bipartitioning strategy
 
   invedlosiz = (tgtedlotax != NULL) ? tgtgrafptr->edgenbr : 0;
   if ((memAllocGroup ((void **) (void *)
-                      &jobtab,     (size_t) (termdomnbr * sizeof (ArchBuildJob)),
+                      &jobtab,     (size_t) (termdomnbr * sizeof (ArchDecoBuildJob)),
                       &actfrontab, (size_t) (termdomnbr * sizeof (Gnum)),
                       &actparttax, (size_t) (termdomnbr * sizeof (GraphPart)),
                       &invedlotax, (size_t) (invedlosiz * sizeof (Gnum)), NULL) == NULL) ||
       ((mappdat.parttax = memAlloc (tgtgrafptr->vertnbr * sizeof (ArchDomNum))) == NULL) || /* Final mapping array is for all graph vertices */
       ((mappdat.domntab = memAlloc (termdomnbr * sizeof (ArchDom)))             == NULL)) {
-    errorPrint ("archBuild: out of memory (1)");
+    errorPrint ("archDecoBuild: out of memory (1)");
     if (jobtab != NULL) {
       memFree (jobtab);
       if (mappdat.parttax != NULL)
@@ -175,91 +177,34 @@ const Strat * const         mapstrat)             /*+ Bipartitioning strategy
     }
     return (1);
   }
-  memSet (mappdat.parttax, 0, termdomnbr * sizeof (ArchDomNum));
+  memSet (mappdat.parttax, 0, tgtgrafptr->vertnbr * sizeof (ArchDomNum));
   actparttax      -= tgtgrafptr->baseval;
-  mappdat.baseval  = tgtgrafptr->baseval;
-  mappdat.vertnbr  = termdomnbr;
+  mappdat.flagval  = MAPPINGFREEPART | MAPPINGFREEDOMN;
+  mappdat.grafptr  = tgtgrafptr;
+  mappdat.archptr  = &archdat;
   mappdat.parttax -= tgtgrafptr->baseval;
   mappdat.domnmax  = termdomnbr;
 
-  archInit (&mappdat.archdat);                    /* Initialize terminal architecture */
-  mappdat.archdat.class = archClass ("varcmplt"); /* Set architecture class           */
-  archDomFrst (&mappdat.archdat, &mappdat.domntab[0]); /* Get initial domain          */
+  archInit (&archdat);                            /* Initialize terminal architecture */
+  archdat.class = archClass ("varcmplt");         /* Set architecture class           */
+  archDomFrst (&archdat, &mappdat.domntab[0]);    /* Get initial domain               */
   mappdat.domnnbr = 1;
 
   jobtab[0].domnum = 0;                           /* All vertices mapped to first domain  */
   if ((tgtlistptr != NULL) && (tgtlistptr->vnumtab != NULL)) /* If vertex list given      */
-    graphInduceList (tgtgrafptr, tgtlistptr, &jobtab[0].grafdat); /* Restrict initial job */
+    graphInduceList (tgtgrafptr, tgtlistptr->vnumnbr, tgtlistptr->vnumtab, &jobtab[0].grafdat); /* Restrict initial job */
   else {                                          /* If no vertex list given              */
     memCpy (&jobtab[0].grafdat, tgtgrafptr, sizeof (Graph)); /* Job takes whole graph     */
     jobtab[0].grafdat.flagval &= ~GRAPHFREETABS;  /* Graph is a clone                     */
+    jobtab[0].grafdat.vnumtax  = NULL;            /* Assume we have no vertex index array */
   }
 
-  if (tgtedlotax != NULL) {                       /* If architecture graph has edge loads */
-    Gnum                  vertnum;
-    Gnum                  vertnnd;
-    Gnum                  edlomin;
-    Gnum                  edlomax;
-    float                 prodval;
-
-    const Gnum * restrict const indverttax = jobtab[0].grafdat.verttax;
-    const Gnum * restrict const indvendtax = jobtab[0].grafdat.vendtax;
-    const Gnum * restrict const indedlotax = jobtab[0].grafdat.edlotax; /* Point to possibly induced original edge array */
-
-    invedlotax -= tgtgrafptr->baseval;            /* Base inversed edge load array */
-
-    edlomin = GNUMMAX;
-    edlomax = 1;
-    for (vertnum = jobtab[0].grafdat.baseval, vertnnd = jobtab[0].grafdat.vertnnd; /* Handle non-compact graphs as well as compact graphs */
-         vertnum < vertnnd; vertnum ++) {
-      Gnum                  edgenum;
-      Gnum                  edgennd;
-
-      for (edgenum = indverttax[vertnum], edgennd = indvendtax[vertnum];
-           edgenum < edgennd; edgenum ++) {
-        Gnum                edloval;
-
-        edloval = indedlotax[edgenum];
-        if (edloval < edlomin)
-          edlomin = edloval;
-        if (edloval > edlomax)
-          edlomax = edloval;
-      }
-    }
-
-    prodval = (float) edlomin * (float) edlomax;
-
-    for (vertnum = jobtab[0].grafdat.baseval;
-         vertnum < vertnnd; vertnum ++) {
-      Gnum                  edgenum;
-      Gnum                  edgennd;
-
-      for (edgenum = indverttax[vertnum], edgennd = indvendtax[vertnum];
-           edgenum < edgennd; edgenum ++) {
-        Gnum                edloval;
-
-        edloval = indedlotax[edgenum];
-        if (edloval == edlomin)
-          edloval = edlomax;
-        else if (edloval == edlomax)
-          edloval = edlomin;
-        else
-          edloval = (Gnum) (prodval / (float) edloval + 0.49F);
-#ifdef SCOTCH_DEBUG_ARCH2
-        if ((edloval < edlomin) || (edloval > edlomax)) {
-          errorPrint ("archBuild: internal error (1)");
-          return     (1);
-        }
-#endif /* SCOTCH_DEBUG_ARCH2 */
-        invedlotax[edgenum] = edloval;            /* Write inversed cost in working array */
-      }
-    }
-
+  if (tgtedlotax != NULL) {                       /* If architecture graph has edge loads                     */
+    invedlotax -= tgtgrafptr->baseval;            /* Base inversed edge load array                            */
+    graphIelo (&jobtab[0].grafdat, jobtab[0].grafdat.edlotax, invedlotax); /* Compute inverse of edge loads   */
     jobtab[0].grafdat.edlotax = invedlotax;       /* Replace potentially induced edge array with inversed one */
   }                                               /* Edge array will be freed along with jobtab group leader  */
 
-  mapparttax = mappdat.parttax;
-
   actgrafdat.veextax = NULL;                      /* No external gain array      */
   actgrafdat.parttax = actparttax;                /* Set global auxiliary arrays */
   actgrafdat.frontab = actfrontab;
@@ -275,27 +220,29 @@ const Strat * const         mapstrat)             /*+ Bipartitioning strategy
 
     memCpy (&actgrafdat.s, &joborgptr->grafdat, sizeof (Graph));
     actgrafdat.s.flagval = joborgptr->grafdat.flagval & ~GRAPHFREETABS;
-    bgraphInit2 (&actgrafdat, 1, 1, 1);           /* Create active graph         */
+    bgraphInit2 (&actgrafdat, 1, 1, 1, 0, 0);     /* Create active graph         */
     if (bgraphBipartSt (&actgrafdat, mapstrat) != 0) { /* Perform bipartitioning */
-      errorPrint       ("archBuild: internal error (2)");
-      archBuildJobExit (joborgptr);
-      archBuildJobExit (joblink);
-      mapExit          (&mappdat);
-      memFree          (jobtab);
-      return           (1);
+      errorPrint ("archDecoBuild: internal error");
+      archDecoBuildJobExit (joborgptr);
+      archDecoBuildJobExit (joblink);
+      archExit (&archdat);
+      mapExit  (&mappdat);
+      memFree  (jobtab);
+      return   (1);
     }
     if ((actgrafdat.compsize0 == 0) ||            /* If one of the jobs is empty */
         (actgrafdat.compsize0 == actgrafdat.s.vertnbr)) {
-      errorPrint       ("archBuild: strategy leads to empty domains");
-      graphExit        (&actgrafdat.s);           /* Only free graph part, global arrays kept */
-      archBuildJobExit (joborgptr);
-      archBuildJobExit (joblink);
-      mapExit          (&mappdat);
-      memFree          (jobtab);
-      return           (1);
+      errorPrint ("archDecoBuild: strategy leads to empty domains");
+      graphExit  (&actgrafdat.s);                 /* Only free graph part, global arrays kept */
+      archDecoBuildJobExit (joborgptr);
+      archDecoBuildJobExit (joblink);
+      archExit (&archdat);
+      mapExit  (&mappdat);
+      memFree  (jobtab);
+      return   (1);
     }
 
-    archVcmpltDomBipart ((const ArchVcmplt * const) (void *) &mappdat.archdat, /* Update mapping domains */
+    archVcmpltDomBipart ((const ArchVcmplt * const) (void *) &archdat, /* Update mapping domains */
                          (const ArchVcmpltDom * const) (void *) &mappdat.domntab[joborgptr->domnum],
                          (ArchVcmpltDom * const) (void *) &domsub0,
                          (ArchVcmpltDom * const) (void *) &mappdat.domntab[mappdat.domnnbr]);
@@ -343,10 +290,11 @@ const Strat * const         mapstrat)             /*+ Bipartitioning strategy
   if (memAllocGroup ((void **) (void *)
                      &termverttab, (size_t) (termdomnbr                            * sizeof (ArchDecoTermVert)),
                      &termdisttab, (size_t) (((termdomnbr * (termdomnbr - 1)) / 2) * sizeof (Anum)),
-                     &disttax,     (size_t) (tgtgrafptr->vertnbr                   * sizeof (ArchBuildDistElem)), 
-                     &queutab,     (size_t) (tgtgrafptr->vertnbr                   * sizeof (ArchBuildQueuElem)), NULL) == NULL) {
-    errorPrint ("archBuild: out of memory (2)");
+                     &disttax,     (size_t) (tgtgrafptr->vertnbr                   * sizeof (ArchDecoBuildDistElem)), 
+                     &queutab,     (size_t) (tgtgrafptr->vertnbr                   * sizeof (ArchDecoBuildQueuElem)), NULL) == NULL) {
+    errorPrint ("archDecoBuild: out of memory (2)");
     mapExit    (&mappdat);
+    archExit   (&archdat);
     return     (1);
   }
 
@@ -356,7 +304,7 @@ const Strat * const         mapstrat)             /*+ Bipartitioning strategy
     tgtvertnum = (tgtlistptr != NULL) ? tgtlistptr->vnumtab[termdomnum] : (termdomnum + tgtgrafptr->baseval);
     termverttab[termdomnum].labl = tgtvertnum;
     termverttab[termdomnum].wght = (tgtgrafptr->velotax != NULL) ? tgtgrafptr->velotax[tgtvertnum] : 1;
-    termverttab[termdomnum].num  = archDomNum (&mappdat.archdat, mapDomain (&mappdat, tgtvertnum - tgtgrafptr->baseval));
+    termverttab[termdomnum].num  = archDomNum (&archdat, mapDomain (&mappdat, tgtvertnum - tgtgrafptr->baseval));
     if (termverttab[termdomnum].num > termdommax) /* Find maximum terminal number */
       termdommax = termverttab[termdomnum].num;
   }
@@ -406,10 +354,11 @@ const Strat * const         mapstrat)             /*+ Bipartitioning strategy
         disttax[termverttab[termdomend].labl].distval;
   }
 
-  archDecoArchBuild ((ArchDeco *) (void *) &tgtarchptr->data, termdomnbr, termdommax, termverttab, termdisttab);
+  archDecoArchBuild2 ((ArchDeco *) (void *) &tgtarchptr->data, termdomnbr, termdommax, termverttab, termdisttab);
 
-  memFree (termverttab);                          /* Free group leader */
-  mapExit (&mappdat);
+  memFree  (termverttab);                         /* Free group leader */
+  mapExit  (&mappdat);
+  archExit (&archdat);
 
   return (0);
 }
