@@ -1,4 +1,4 @@
-/* Copyright 2004,2007,2008,2010 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007,2008,2010,2011,2014 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -34,6 +34,7 @@
 /**   NAME       : arch_vcmplt.c                           **/
 /**                                                        **/
 /**   AUTHOR     : Francois PELLEGRINI                     **/
+/**                Sebastien FOURESTIER (v6.0)             **/
 /**                                                        **/
 /**   FUNCTION   : This module handles the variable-sized  **/
 /**                complete graph target architecture.     **/
@@ -62,6 +63,8 @@
 /**                                 to     05 nov 2003     **/
 /**                # Version 5.1  : from : 21 jan 2008     **/
 /**                                 to     11 aug 2010     **/
+/**                # Version 6.0  : from : 14 feb 2011     **/
+/**                                 to     26 aug 2014     **/
 /**                                                        **/
 /************************************************************/
 
@@ -110,8 +113,17 @@ const ArchVcmplt * const    archptr,
 ArchVcmpltDom * const       domptr,
 const ArchDomNum            domnum)
 {
-  if (domnum != ARCHDOMNOTTERM) {                 /* If valid label */
+  Anum                termnum;
+  Anum                termlvl;
+
+  if (domnum != ARCHDOMNOTTERM) {                 /* If valid label     */
+    if (domnum == 0)                              /* Not a legal domain */
+      return (2);
+
     domptr->termnum = domnum;                     /* Set the domain */
+    for (termnum = domnum, termlvl = 0; termnum > 1; termnum >>= 1, termlvl ++) ; /* Compute level */
+    domptr->termlvl = termlvl;                    /* Set level */
+
     return (0);
   }
 
@@ -156,7 +168,8 @@ archVcmpltDomFrst (
 const ArchVcmplt * const        archptr,
 ArchVcmpltDom * restrict const  domptr)
 {
-  domptr->termnum = 1;                            /* First terminal number */
+  domptr->termlvl = 0;                            /* First terminal number */
+  domptr->termnum = 1;
 
   return (0);
 }
@@ -175,11 +188,17 @@ const ArchVcmplt * const        archptr,
 ArchVcmpltDom * restrict const  domptr,
 FILE * const                    stream)
 {
+  Anum                termnum;
+  Anum                termlvl;
+
   if (intLoad (stream, &domptr->termnum) != 1) {
     errorPrint ("archVcmpltDomLoad: bad input");
     return     (1);
   }
 
+  for (termnum = domptr->termnum, termlvl = 0; termnum > 1; termnum >>= 1, termlvl ++) ; /* Compute level */
+  domptr->termlvl = termlvl;
+
   return (0);
 }
 
@@ -219,12 +238,35 @@ const ArchVcmpltDom * const     domptr,
 ArchVcmpltDom * restrict const  dom0ptr,
 ArchVcmpltDom * restrict const  dom1ptr)
 {
+  dom0ptr->termlvl =                              /* Bipartition the domain */
+  dom1ptr->termlvl = domptr->termlvl + 1;
   dom0ptr->termnum = domptr->termnum << 1;
   dom1ptr->termnum = dom0ptr->termnum + 1;
 
   return ((dom1ptr->termnum < domptr->termnum) ? 2 : 0); /* Return error on overflow */
 }
 
+/* This function checks if dom1 is
+** included in dom0.
+** It returns:
+** - 0  : if dom1 is not included in dom0.
+** - 1  : if dom1 is included in dom0.
+** - 2  : on error.
+*/
+
+int
+archVcmpltDomIncl (
+const ArchVcmplt * const    archptr,
+const ArchVcmpltDom * const dom0ptr,
+const ArchVcmpltDom * const dom1ptr)
+{
+  if ((dom1ptr->termlvl >= dom0ptr->termlvl) &&
+      ((dom1ptr->termnum >> (dom1ptr->termlvl - dom0ptr->termlvl)) == dom0ptr->termnum))
+      return (1);
+
+  return (0);
+}
+
 /* This function creates the MPI_Datatype for
 ** variable-sized complete graph domains.
 ** It returns:
@@ -238,7 +280,7 @@ archVcmpltDomMpiType (
 const ArchVcmplt * const      archptr,
 MPI_Datatype * const          typeptr)
 {
-  *typeptr = ANUM_MPI;
+  MPI_Type_contiguous (2, ANUM_MPI, typeptr);
 
   return (0);
 }
