@@ -1,4 +1,4 @@
-/* Copyright 2004,2007,2008 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007,2008,2011,2014,2015,2018 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -53,6 +53,8 @@
 /**                                 to     05 nov 2003     **/
 /**                # Version 5.1  : from : 21 jan 2008     **/
 /**                                 to     21 jan 2008     **/
+/**                # Version 6.0  : from : 14 feb 2011     **/
+/**                                 to     28 may 2018     **/
 /**                                                        **/
 /************************************************************/
 
@@ -60,6 +62,9 @@
 **  The type and structure definitions.
 */
 
+#ifndef ARCH_VCMPLT_H_STRUCT
+#define ARCH_VCMPLT_H_STRUCT
+
 /*+ The variable-sized complete graph bipartitioning definitions. +*/
 
 typedef struct ArchVcmplt_ {
@@ -67,20 +72,28 @@ typedef struct ArchVcmplt_ {
 } ArchVcmplt;
 
 typedef struct ArchVcmpltDom_ {
+  Anum                      termlvl;              /*+ Terminal depth  +*/
   Anum                      termnum;              /*+ Terminal number +*/
 } ArchVcmpltDom;
 
+#endif /* ARCH_VCMPLT_H_STRUCT */
+
 /*
 **  The function prototypes.
 */
 
-#ifndef ARCH_VCMPLT
-#define static
-#endif
+#ifndef ARCH_NOPROTO
+#ifndef ARCH_VCMPLT_H_PROTO
+#define ARCH_VCMPLT_H_PROTO
 
 #define archVcmpltArchLoad          NULL
 #define archVcmpltArchSave          NULL
 #define archVcmpltArchFree          NULL
+
+#define archVcmpltMatchInit         NULL
+#define archVcmpltMatchExit         NULL
+#define archVcmpltMatchMate         NULL
+
 ArchDomNum                  archVcmpltDomNum    (const ArchVcmplt * const, const ArchVcmpltDom * const);
 int                         archVcmpltDomTerm   (const ArchVcmplt * const, ArchVcmpltDom * restrict const, const ArchDomNum);
 Anum                        archVcmpltDomSize   (const ArchVcmplt * const, const ArchVcmpltDom * const);
@@ -90,8 +103,10 @@ int                         archVcmpltDomFrst   (const ArchVcmplt * const, ArchV
 int                         archVcmpltDomLoad   (const ArchVcmplt * const, ArchVcmpltDom * const, FILE * const);
 int                         archVcmpltDomSave   (const ArchVcmplt * const, const ArchVcmpltDom * const, FILE * const);
 int                         archVcmpltDomBipart (const ArchVcmplt * const, const ArchVcmpltDom * const, ArchVcmpltDom * restrict const, ArchVcmpltDom * restrict const);
+int                         archVcmpltDomIncl   (const ArchVcmplt * const, const ArchVcmpltDom * const, const ArchVcmpltDom * const);
 #ifdef SCOTCH_PTSCOTCH
 int                         archVcmpltDomMpiType (const ArchVcmplt * const, MPI_Datatype * const);
 #endif /* SCOTCH_PTSCOTCH */
 
-#undef static
+#endif /* ARCH_VCMPLT_H_PROTO */
+#endif /* ARCH_NOPROTO        */
