@@ -1,4 +1,4 @@
-/* Copyright 2007-2012 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2007-2014,2018 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -42,7 +42,7 @@
 /**   DATES      : # Version 5.1  : from : 30 oct 2007     **/
 /**                                 to   : 14 apr 2011     **/
 /**              : # Version 6.0  : from : 11 sep 2011     **/
-/**                                 to   : 18 sep 2012     **/
+/**                                 to   : 07 jun 2018     **/
 /**                                                        **/
 /************************************************************/
 
@@ -88,17 +88,29 @@ const BdgraphBipartMlParam * const    paraptr)    /*+ Method parameters
 {
   int                 foldval;
 
-  foldval = 1;
-  if ((paraptr->duplvlmax > -1) &&                /* duplvlmax can allow fold dup */
-      (paraptr->duplvlmax < finegrafptr->levlnum + 1))
-    foldval = 0;
-  else if (paraptr->duplvlmax < -1)               /* duplvlmax can allow only fold */
-    foldval = (-(paraptr->duplvlmax + 1) < (finegrafptr->levlnum + 1)) ? 0 : -1;
+  switch (paraptr->foldval) {
+    case 0 :
+      foldval = DGRAPHCOARSENNONE;
+      break;
+    case 1 :
+      foldval = DGRAPHCOARSENFOLD;
+      break;
+    case 2 :
+      foldval = DGRAPHCOARSENFOLDDUP;
+      break;
+#ifdef SCOTCH_DEBUG_BGRAPH2
+    default :
+      errorPrint ("bdgraphBipartMlCoarsen: invalid parameter");
+      return     (1);
+#endif /* SCOTCH_DEBUG_BDGRAPH2 */
+  }
+  if ((finegrafptr->s.vertglbnbr / finegrafptr->s.procglbnbr) > paraptr->foldmax) /* If no need to fold */
+    foldval = DGRAPHCOARSENNONE;
 
   *coarmultptr = NULL;                            /* Let the routine create the multinode array */
   dgraphInit (&coargrafptr->s, finegrafptr->s.proccomm); /* Re-use fine graph communicator      */
   if (dgraphCoarsen (&finegrafptr->s, &coargrafptr->s, coarmultptr, paraptr->passnbr,
-                     paraptr->coarnbr, 0, foldval, paraptr->dupmax, paraptr->coarrat) != 0)
+                     paraptr->coarnbr, paraptr->coarrat, foldval) != 0)
     return (1);                                   /* Return if coarsening failed */
 
   *coarmultptr -= coargrafptr->s.baseval;         /* Base pointer to multinode array    */
@@ -146,9 +158,9 @@ const BdgraphBipartMlParam * const    paraptr)    /*+ Method parameters
   coargrafptr->compglbload0avg  = finegrafptr->compglbload0avg;
   coargrafptr->commglbloadextn0 = finegrafptr->commglbloadextn0;
   coargrafptr->commglbgainextn0 = finegrafptr->commglbgainextn0;
-  coargrafptr->domdist          = finegrafptr->domdist;
-  coargrafptr->domwght[0]       = finegrafptr->domwght[0];
-  coargrafptr->domwght[1]       = finegrafptr->domwght[1];
+  coargrafptr->domndist         = finegrafptr->domndist;
+  coargrafptr->domnwght[0]      = finegrafptr->domnwght[0];
+  coargrafptr->domnwght[1]      = finegrafptr->domnwght[1];
   coargrafptr->levlnum          = finegrafptr->levlnum + 1;
 
   return (0);
@@ -222,7 +234,6 @@ const DgraphCoarsenMulti * restrict const coarmulttax) /*+ Multinode array +*/
   Gnum                            finefronlocnum;
   Gnum                            fineedlolocval;
   Gnum                            finevertlocadj; /* Global vertex adjustment                            */
-  Gnum                            finevertlocnum;
   Gnum                            finevertlocnnd; /* Index for frontier array fronloctab                 */
   Gnum                            finecomplocsize1;
   Gnum                            finecomplocload1;
@@ -250,6 +261,9 @@ const DgraphCoarsenMulti * restrict const coarmulttax) /*+ Multinode array +*/
   GraphPart * restrict            coarpartgsttax;
   GraphPart * restrict            finepartgsttax;
   Gnum * restrict                 finefronloctab;
+#ifdef SCOTCH_DEBUG_BDGRAPH2
+  Gnum                            finevertlocnum;
+#endif /* SCOTCH_DEBUG_BDGRAPH2 */
 
   const int                   fineprocglbnbr = finegrafptr->s.procglbnbr;
   const Gnum * restrict const fineprocvrttab = finegrafptr->s.procvrttab;
@@ -291,6 +305,13 @@ const DgraphCoarsenMulti * restrict const coarmulttax) /*+ Multinode array +*/
 
     bdgraphZero (finegrafptr);                    /* Assign all vertices to part 0 */
 
+#ifdef SCOTCH_DEBUG_BDGRAPH2
+    if (bdgraphCheck (finegrafptr) != 0) {
+      errorPrint ("bdgraphBipartMlUncoarsen: inconsistent graph data (1)");
+      return     (1);
+    }
+#endif /* SCOTCH_DEBUG_BDGRAPH2 */
+
     return (0);
   }
 
@@ -410,7 +431,7 @@ const DgraphCoarsenMulti * restrict const coarmulttax) /*+ Multinode array +*/
 #ifdef SCOTCH_DEBUG_BDGRAPH2
           if (finepartgsttax[finevertlocnum] != ((GraphPart) ~0)) {
             errorPrint ("bdgraphBipartMlUncoarsen: internal error (1)");
-            return (1);
+            return     (1);
           }
 #endif /* SCOTCH_DEBUG_BDGRAPH2 */
           finepartgsttax[finevertlocnum] = (coarpartval & 1);
@@ -469,7 +490,7 @@ const DgraphCoarsenMulti * restrict const coarmulttax) /*+ Multinode array +*/
   }
 
   if (MPI_Alltoall (vsndcnttab, 1, MPI_INT, vrcvcnttab, 1, MPI_INT, finegrafptr->s.proccomm) != MPI_SUCCESS) {
-    errorPrint ("bdgraphBipartMlUncoarsen: communication error (3)");
+    errorPrint ("bdgraphBipartMlUncoarsen: communication error (6)");
     return     (1);
   }
 
@@ -489,7 +510,7 @@ const DgraphCoarsenMulti * restrict const coarmulttax) /*+ Multinode array +*/
   }
 #ifdef SCOTCH_DEBUG_BDGRAPH1                      /* Communication cannot be overlapped by a useful one */
   if (MPI_Allreduce (&reduloctab[5], &reduglbtab[5], 1, GNUM_MPI, MPI_SUM, finegrafptr->s.proccomm) != MPI_SUCCESS)  {
-    errorPrint ("bdgraphBipartMlUncoarsen: communication error (4)");
+    errorPrint ("bdgraphBipartMlUncoarsen: communication error (7)");
     return     (1);
   }
 #else /* SCOTCH_DEBUG_BDGRAPH1 */
@@ -549,8 +570,8 @@ const DgraphCoarsenMulti * restrict const coarmulttax) /*+ Multinode array +*/
 #ifdef SCOTCH_DEBUG_BDGRAPH2
     if (((partval < 3) && (vsndidxtab[4 * procngbnum + partval] >= vsndidxtab[4 * procngbnum + partval + 1])) ||
         (vsndidxtab[4 * procngbnum + partval] >= (vsnddsptab[procngbnum] + vsndcnttab[procngbnum]))) {
-      errorPrint ("bdgraphBipartMlUncoarsen: internal error (3)");
-      return (1);
+      errorPrint ("bdgraphBipartMlUncoarsen: internal error (2)");
+      return     (1);
     }
 #endif /* SCOTCH_DEBUG_BDGRAPH2 */
     vsnddattab[vsndidxtab[4 * procngbnum + partval] ++] = vertglbend; /* Pack vertex in proper sub-array */
@@ -558,8 +579,8 @@ const DgraphCoarsenMulti * restrict const coarmulttax) /*+ Multinode array +*/
 
   if (MPI_Alltoallv (vsnddattab, vsndcnttab, vsnddsptab, GNUM_MPI, 
                      vrcvdattab, vrcvcnttab, vrcvdsptab, GNUM_MPI, finegrafptr->s.proccomm) != MPI_SUCCESS) {
-    errorPrint ("bdgraphBipartMlUncoarsen: communication error (5)");
-    return (1);
+    errorPrint ("bdgraphBipartMlUncoarsen: communication error (8)");
+    return     (1);
   }
     
   for (procnum = 0; procnum < fineprocglbnbr; ++ procnum) { /* Update local ones from the buffer for receiving data */
@@ -578,8 +599,8 @@ const DgraphCoarsenMulti * restrict const coarmulttax) /*+ Multinode array +*/
       finevertlocnum = vrcvdattab[vrcvidxnum] - finevertlocadj;
 #ifdef SCOTCH_DEBUG_BDGRAPH2
       if ((finevertlocnum < baseval) || (finevertlocnum >= finevertlocnnd)) {
-        errorPrint ("bdgraphBipartMlUncoarsen: internal error (4)");
-        return (1);
+        errorPrint ("bdgraphBipartMlUncoarsen: internal error (3)");
+        return     (1);
       }
 #endif /* SCOTCH_DEBUG_BDGRAPH2 */
       finepartgsttax[finevertlocnum] = 0;
@@ -592,8 +613,8 @@ const DgraphCoarsenMulti * restrict const coarmulttax) /*+ Multinode array +*/
       finevertlocnum = vrcvdattab[vrcvidxnum] - finevertlocadj;
 #ifdef SCOTCH_DEBUG_BDGRAPH2
       if ((finevertlocnum < baseval) || (finevertlocnum >= finevertlocnnd)) {
-        errorPrint ("bdgraphBipartMlUncoarsen: internal error (5)");
-        return (1);
+        errorPrint ("bdgraphBipartMlUncoarsen: internal error (4)");
+        return     (1);
       }
 #endif /* SCOTCH_DEBUG_BDGRAPH2 */
       finepartgsttax[finevertlocnum] = 1;
@@ -615,7 +636,7 @@ const DgraphCoarsenMulti * restrict const coarmulttax) /*+ Multinode array +*/
       finevertlocnum = vrcvdattab[vrcvidxnum] - finevertlocadj;
 #ifdef SCOTCH_DEBUG_BDGRAPH2
       if ((finevertlocnum < baseval) || (finevertlocnum >= finevertlocnnd)) {
-        errorPrint ("bdgraphBipartMlUncoarsen: internal error (6)");
+        errorPrint ("bdgraphBipartMlUncoarsen: internal error (5)");
         return     (1);
       }
 #endif /* SCOTCH_DEBUG_BDGRAPH2 */
@@ -630,7 +651,7 @@ const DgraphCoarsenMulti * restrict const coarmulttax) /*+ Multinode array +*/
       finevertlocnum = vrcvdattab[vrcvidxnum] - finevertlocadj;
 #ifdef SCOTCH_DEBUG_BDGRAPH2
       if ((finevertlocnum < baseval) || (finevertlocnum >= finevertlocnnd)) {
-        errorPrint ("bdgraphBipartMlUncoarsen: internal error (7)");
+        errorPrint ("bdgraphBipartMlUncoarsen: internal error (6)");
         return     (1);
       }
 #endif /* SCOTCH_DEBUG_BDGRAPH2 */
@@ -651,7 +672,7 @@ const DgraphCoarsenMulti * restrict const coarmulttax) /*+ Multinode array +*/
 #ifdef SCOTCH_DEBUG_BDGRAPH2
   for (finevertlocnum = baseval; finevertlocnum < finevertlocnnd; finevertlocnum ++) {
     if (finepartgsttax[finevertlocnum] == ((GraphPart) ~0)) {
-      errorPrint ("bdgraphBipartMlUncoarsen: internal error (8)");
+      errorPrint ("bdgraphBipartMlUncoarsen: internal error (7)");
       return     (1);
     }
   }
@@ -705,16 +726,13 @@ const DgraphCoarsenMulti * restrict const coarmulttax) /*+ Multinode array +*/
   finegrafptr->compglbload0dlt = finegrafptr->compglbload0 - finegrafptr->compglbload0avg;
   finegrafptr->compglbsize0    = reduglbtab[1];
   finegrafptr->fronglbnbr      = reduglbtab[2];
-  finegrafptr->commglbload     = ((reduglbtab[3] / 2) * finegrafptr->domdist + reduglbtab[4] + finegrafptr->commglbloadextn0);
+  finegrafptr->commglbload     = ((reduglbtab[3] / 2) * finegrafptr->domndist + reduglbtab[4] + finegrafptr->commglbloadextn0);
   finegrafptr->commglbgainextn = reduglbtab[5];
   finegrafptr->bbalglbval      = coargrafptr->bbalglbval;
 
-  if (coargrafptr->s.procglbnbr != 0)
-    memFree ((void *) (coarmulttax + coargrafptr->s.baseval));
-
 #ifdef SCOTCH_DEBUG_BDGRAPH2
   if (bdgraphCheck (finegrafptr) != 0) {
-    errorPrint ("bdgraphBipartMlUncoarsen: inconsistent graph data");
+    errorPrint ("bdgraphBipartMlUncoarsen: inconsistent graph data (2)");
     return     (1);
   }
 #endif /* SCOTCH_DEBUG_BDGRAPH2 */
@@ -749,6 +767,7 @@ const BdgraphBipartMlParam * const  paraptr)      /* Method parameters */
     return (o);
   }
 
+  coarmulttax = NULL;                             /* Assume multinode array is not allocated */
   if (bdgraphBipartMlCoarsen (grafptr, &coargrafdat, &coarmulttax, paraptr) == 0) {
     o = (coargrafdat.s.procglbnbr == 0) ? 0 : bdgraphBipartMl2 (&coargrafdat, paraptr); /* Apply recursion on coarsened graph if it exists */
 
@@ -761,6 +780,8 @@ const BdgraphBipartMlParam * const  paraptr)      /* Method parameters */
     }
 
     bdgraphExit (&coargrafdat);
+    if (coarmulttax != NULL)                      /* If multinode array has been allocated */
+      memFree (coarmulttax + grafptr->s.baseval); /* Free array                            */
 
     if (o == 0)                                   /* If multi-level failed, apply low strategy as fallback */
       return (o);
