@@ -1,4 +1,4 @@
-/* Copyright 2004,2007-2010 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007-2016,2018 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -38,6 +38,7 @@
 /**                Pascal HENON                            **/
 /**                Pierre RAMET                            **/
 /**                Cedric CHEVALIER (v5.0)                 **/
+/**                Sebastien FOURESTIER (v6.0)             **/
 /**                                                        **/
 /**   FUNCTION   : Part of a parallel direct block solver. **/
 /**                These lines are the common data         **/
@@ -51,6 +52,8 @@
 /**                                 to   : 01 jul 2008     **/
 /**                # Version 5.1  : from : 09 nov 2008     **/
 /**                                 to   : 23 nov 2010     **/
+/**                # Version 6.0  : from : 03 mar 2011     **/
+/**                                 to     14 jul 2018     **/
 /**                                                        **/
 /************************************************************/
 
@@ -63,8 +66,12 @@
 #ifndef _XOPEN_SOURCE
 #define _XOPEN_SOURCE               600
 #endif /* _XOPEN_SOURCE */
+#ifndef __USE_XOPEN2K
+#define __USE_XOPEN2K                             /* For POSIX pthread_barrier_t */
+#endif /* __USE_XOPEN2K */
 
 #include            <ctype.h>
+#include            <fcntl.h>                     /* Fow Windows _pipe () call */
 #include            <math.h>
 #include            <memory.h>
 #include            <stdio.h>
@@ -89,19 +96,23 @@
 #include            <sys/resource.h>
 #endif /* ((defined COMMON_TIMING_OLD) || (defined HAVE_SYS_RESOURCE_H)) */
 #if ((defined COMMON_WINDOWS) || (defined HAVE_WINDOWS_H))
+#include            <io.h>                        /* For _pipe () */
+#include            <stddef.h>                    /* For intptr_t */
 #include            <windows.h>
 #endif /* ((defined COMMON_WINDOWS) || (defined HAVE_WINDOWS_H)) */
 #if ((! defined COMMON_WINDOWS) && (! defined HAVE_NOT_UNISTD_H))
 #include            <unistd.h>
 #endif /* ((! defined COMMON_WINDOWS) && (! defined HAVE_NOT_UNISTD_H)) */
 
-#ifdef SCOTCH_PTSCOTCH
+#ifdef COMMON_MPI
 #include            <mpi.h>
-#endif /* SCOTCH_PTSCOTCH */
+#endif /* COMMON_MPI */
 
-#if ((defined COMMON_PTHREAD) || (defined SCOTCH_PTHREAD))
+#ifdef COMMON_PTHREAD
 #include            <pthread.h>
-#endif /* ((defined COMMON_PTHREAD) || (defined SCOTCH_PTHREAD)) */
+#else /* COMMON_PTHREAD */
+#include            <sys/wait.h>
+#endif /* COMMON_PTHREAD */
 
 /*
 **  Working definitions.
@@ -110,16 +121,16 @@
 #ifdef COMMON_MEMORY_TRACE
 #define memAlloc(size)              memAllocRecord ((size) | 8)
 #define memRealloc(ptr,size)        memReallocRecord ((ptr), ((size) | 8))
-#define memFree(ptr)                (memFreeRecord ((void *) (ptr)), 0)
+#define memFree(ptr)                memFreeRecord ((void *) (ptr))
 #else /* COMMON_MEMORY_TRACE */
-#define memAlloc(size)              malloc((size) | 8) /* For platforms which return NULL for malloc(0) */
-#define memRealloc(ptr,size)        realloc((ptr),((size) | 8))
-#define memFree(ptr)                (free ((char *) (ptr)), 0)
+#define memAlloc(size)              malloc ((size) | 8) /* For platforms which return NULL for malloc(0) */
+#define memRealloc(ptr,size)        realloc ((ptr),((size) | 8))
+#define memFree(ptr)                free ((char *) (ptr))
 #endif /* COMMON_MEMORY_TRACE */
 
-#define memSet(ptr,val,siz)         memset((ptr),(val),(siz))
-#define memCpy(dst,src,siz)         memcpy((dst),(src),(siz))
-#define memMov(dst,src,siz)         memmove((dst),(src),(siz))
+#define memSet(ptr,val,siz)         memset ((void *) (ptr), (val), (siz))
+#define memCpy(dst,src,siz)         memcpy ((void *) (dst), (void *) (src), (siz))
+#define memMov(dst,src,siz)         memmove ((void *) (dst), (void *) (src), (siz))
 
 #define MIN(x,y)                    (((x) < (y)) ? (x) : (y))
 #define MAX(x,y)                    (((x) < (y)) ? (y) : (x))
@@ -130,29 +141,43 @@
 **  Handling of generic types.
 */
 
+#if (((defined __STDC_VERSION__) && (__STDC_VERSION__ >= 199901L)) || (defined HAVE_UINT_T))
+#define UINT32                      uint32_t
+#else /* (((defined __STDC_VERSION__) && (__STDC_VERSION__ >= 199901L)) || (defined HAVE_UINT_T)) */
+#define UINT32                      u_int32_t
+#endif /* (((defined __STDC_VERSION__) && (__STDC_VERSION__ >= 199901L)) || (defined HAVE_UINT_T)) */
+
 #ifndef INT                                       /* If type not externally overriden */
 #ifdef INTSIZE32
 #define INT                         int32_t
-#define UINT                        u_int32_t
+#define UINT                        UINT32
 #define COMM_INT                    MPI_INTEGER4
 #define INTSTRING                   "%d"
+#define UINTSTRING                  "%u"
 #else /* INTSIZE32 */
 #ifdef INTSIZE64
 #define INT                         int64_t
+#if (((defined __STDC_VERSION__) && (__STDC_VERSION__ >= 199901L)) || (defined HAVE_UINT_T))
+#define UINT                        uint64_t
+#else /* (((defined __STDC_VERSION__) && (__STDC_VERSION__ >= 199901L)) || (defined HAVE_UINT_T)) */
 #define UINT                        u_int64_t
+#endif /* (((defined __STDC_VERSION__) && (__STDC_VERSION__ >= 199901L)) || (defined HAVE_UINT_T)) */
 #define COMM_INT                    MPI_LONG_LONG
 #define INTSTRING                   "%lld"
+#define UINTSTRING                  "%llu"
 #else /* INTSIZE64 */
 #ifdef LONG                                       /* Better not use it */
 #define INT                         long          /* Long integer type */
 #define UINT                        unsigned long
 #define COMM_INT                    MPI_LONG
 #define INTSTRING                   "%ld"
+#define UINTSTRING                  "%lu"
 #else /* LONG */
 #define INT                         int           /* Default integer type */
 #define UINT                        unsigned int
 #define COMM_INT                    MPI_INT       /* Generic MPI integer type */
 #define INTSTRING                   "%d"
+#define UINTSTRING                  "%u"
 #endif /* LONG      */
 #endif /* INTSIZE64 */
 #endif /* INTSIZE32 */
@@ -185,6 +210,19 @@
 #endif /* COMM_BYTE */
 #define COMM_PART                   COMM_BYTE
 
+/*
+**  Handling of pseudo-random numbers.
+*/
+
+/* The pseudo-random state structure. It is
+   based on a Mersenne twister generator, also
+   referred to as MT19937. */
+
+typedef struct IntRandState_ {
+  UINT32                    randtab[624];         /* State vector */
+  int                       randnum;              /* Index value  */
+} IntRandState;
+
 /*
 **  Handling of flag arrays.
 */
@@ -203,16 +241,108 @@ typedef struct Clock_ {
   double                    time[2];              /*+ The start and accumulated times +*/
 } Clock;
 
+/*
+**  Handling of Windows constructs.
+*/
+
+#if defined COMMON_WINDOWS
+#define pipe(fd)                    _pipe (fd, 32768, O_BINARY)
+#endif /* COMMON_WINDOWS */
+
+/*
+**  Handling of threads.
+*/
+
+/** The thread creation flags **/
+
+#define THREADNONE                  0x0000        /* Thread capabilities */
+
+#define THREADHASBARRIER            0x0001
+
+#define THREADCANBARRIER            THREADHASBARRIER
+#define THREADCANSCAN               THREADHASBARRIER
+#define THREADCANREDUCE             THREADHASBARRIER
+
+/** The thread barrier structure and routines **/
+
+#ifdef COMMON_PTHREAD_BARRIER
+
+#ifndef PTHREAD_BARRIER_SERIAL_THREAD
+#define PTHREAD_BARRIER_SERIAL_THREAD -1
+#endif /* PTHREAD_BARRIER_SERIAL_THREAD */
+
+typedef struct ThreadBarrier_ {
+  int                       thrdnbr;              /*+ Number of threads to wait for       +*/
+  volatile int              thrdcur;              /*+ Number of threads currently blocked +*/
+  volatile int              instnum;              /*+ Number of barrier instance          +*/
+  pthread_mutex_t           mutedat;
+  pthread_cond_t            conddat;
+} ThreadBarrier;
+
+int                         threadBarrierDestroy (ThreadBarrier *);
+int                         threadBarrierInit   (ThreadBarrier *, void *, int); /* Thread attribute not used */
+int                         threadBarrierWait   (ThreadBarrier *);
+
+#else /* COMMON_PTHREAD_BARRIER */
+
+#define ThreadBarrier               pthread_barrier_t
+
+#define threadBarrierDestroy        pthread_barrier_destroy
+#define threadBarrierInit           pthread_barrier_init
+#define threadBarrierWait           pthread_barrier_wait
+
+#endif /* COMMON_PTHREAD_BARRIER */
+
+#define threadBarrier(t)            threadBarrierWait (&(((ThreadGroupHeader *) (((ThreadHeader *) (void *) (t))->grouptr))->barrdat))
+
+/** The thread service routines auxiliary function types **/
+
+typedef int (* ThreadLaunchJoinFunc) (void * const, void * const);
+typedef int (* ThreadLaunchStartFunc) (void * const);
+typedef void (* ThreadReduceFunc) (void * const, void * const, void * const);
+typedef void (* ThreadScanFunc)   (void * const, void * const, void * const, const int);
+
+/** The thread group header block. **/
+
+typedef struct ThreadGroupHeader_ {
+#ifdef COMMON_PTHREAD
+  int                       flagval;              /*+ Thread block flags       +*/
+  size_t                    datasiz;              /*+ Size of data array cell  +*/
+  int                       thrdnbr;              /*+ Number of threads        +*/
+  ThreadLaunchStartFunc     stafptr;              /*+ Pointer to start routine +*/
+  ThreadLaunchJoinFunc      joifptr;              /*+ Pointer to join routine  +*/
+  ThreadBarrier             barrdat;              /*+ Barrier data structure   +*/
+#endif /* COMMON_PTHREAD */
+} ThreadGroupHeader;
+
+/** The thread header block. **/
+
+typedef struct ThreadHeader_ {
+  void *                    grouptr;              /*+ Pointer to thread group +*/
+#ifdef COMMON_PTHREAD
+  pthread_t                 thidval;              /*+ Thread ID               +*/
+  int                       thrdnum;              /*+ Thread instance number  +*/
+#endif /* COMMON_PTHREAD */
+} ThreadHeader;
+
 /*
 **  Handling of files.
 */
 
+/** The file flags **/
+
+#define FILEMODE                    0x0001
+#define FILEMODER                   0x0000
+#define FILEMODEW                   0x0001
+#define FILEFREENAME                0x0002
+
 /** The file structure. **/
 
 typedef struct File_ {
-  char *                    name;                 /*+ File name    +*/
-  FILE *                    pntr;                 /*+ File pointer +*/
-  char *                    mode;                 /*+ Opening mode +*/
+  int                       flagval;              /*+ File mode            +*/
+  char *                    nameptr;              /*+ File name            +*/
+  FILE *                    fileptr;              /*+ File pointer         +*/
+  void *                    compptr;              /*+ (Un)compression data +*/
 } File;
 
 /*
@@ -226,19 +356,22 @@ void *                      memOffset           (void *, ...);
 void *                      memAllocRecord      (size_t);
 void *                      memReallocRecord    (void * const, size_t);
 void                        memFreeRecord       (void * const);
-size_t                      memMax              ();
+IDX                         memCur              (); /* What is internally an intptr_t has to be turned into an interface type */
+IDX                         memMax              ();
 #endif /* COMMON_MEMORY_TRACE */
 
 void                        usagePrint          (FILE * const, const char (* []));
 
+void                        fileBlockInit       (File * const, const int);
 int                         fileBlockOpen       (File * const, const int);
 int                         fileBlockOpenDist   (File * const, const int, const int, const int, const int);
 void                        fileBlockClose      (File * const, const int);
-FILE *                      fileCompress        (FILE * const, const int);
+int                         fileCompress        (File * const, const int);
+void                        fileCompressExit    (File * const);
 int                         fileCompressType    (const char * const);
-FILE *                      fileUncompress      (FILE * const, const int);
-int                         fileUncompressType  (const char * const);
-int                         fileNameDistExpand  (char ** const, const int, const int, const int);
+int                         fileDecompress      (File * const, const int);
+int                         fileDecompressType  (const char * const);
+char *                      fileNameDistExpand  (char * const, const int, const int);
 
 void                        errorProg           (const char * const);
 void                        errorPrint          (const char * const, ...);
@@ -248,15 +381,22 @@ int                         intLoad             (FILE * const, INT * const);
 int                         intSave             (FILE * const, const INT);
 void                        intAscn             (INT * const, const INT, const INT);
 void                        intPerm             (INT * const, const INT);
-void                        intRandReset        (void);
 void                        intRandInit         (void);
-INT                         intRandVal          (INT);
+int                         intRandLoad         (FILE * const);
+void                        intRandProc         (int);
+void                        intRandReset        (void);
+int                         intRandSave         (FILE * const);
+void                        intRandSeed         (INT);
+#ifndef COMMON_RANDOM_SYSTEM
+UINT                        intRandVal          (UINT);
+#endif /* COMMON_RANDOM_SYSTEM */
 void                        intSort1asc1        (void * const, const INT);
 void                        intSort2asc1        (void * const, const INT);
 void                        intSort2asc2        (void * const, const INT);
 void                        intSort3asc1        (void * const, const INT);
 void                        intSort3asc2        (void * const, const INT);
 INT                         intSearchDicho      (const INT * const, const INT, const INT, const INT);
+INT                         intGcd              (INT, INT);
 
 void                        clockInit           (Clock * const);
 void                        clockStart          (Clock * const);
@@ -266,6 +406,12 @@ double                      clockGet            (void);
 
 void                        stringSubst         (char * const, const char * const, const char * const);
 
+#ifdef COMMON_PTHREAD
+int                         threadLaunch        (void * const, void * const, const size_t, int (*) (void *), int (*) (void *, void *), const int, const int);
+void                        threadReduce        (void * const, void * const, ThreadReduceFunc const, const int);
+void                        threadScan          (void * const, void * const, ThreadScanFunc const);
+#endif /* COMMON_PTHREAD */
+
 /*
 **  Macro definitions.
 */
@@ -275,26 +421,34 @@ void                        stringSubst         (char * const, const char * cons
 #define clockStop(clk)              ((clk)->time[1] += (clockGet () - (clk)->time[0]))
 #define clockVal(clk)               ((clk)->time[1])
 
+#define fileBlockFile(b,i)          ((b)[i].fileptr)
+#define fileBlockMode(b,i)          ((b)[i].modeptr)
+#define fileBlockName(b,i)          ((b)[i].nameptr)
+
+#ifdef COMMON_RANDOM_SYSTEM
 #ifdef COMMON_RANDOM_RAND
-#define intRandVal(ival)            ((INT) (((UINT) rand ()) % ((UINT) (ival))))
+#define intRandVal(ival)            ((UINT) (((UINT) rand ()) % ((UINT) (ival))))
 #else /* COMMON_RANDOM_RAND */
-#define intRandVal(ival)            ((INT) (((UINT) random ()) % ((UINT) (ival))))
+#define intRandVal(ival)            ((UINT) (((UINT) random ()) % ((UINT) (ival))))
 #endif /* COMMON_RANDOM_RAND */
+#endif /* COMMON_RANDOM_SYSTEM */
 
 #define DATASIZE(n,p,i)             ((INT) (((n) + ((p) - 1 - (i))) / (p)))
+#define DATASCAN(n,p,i)             ((i) * ((INT) (n) / (INT) (p)) + (((i) > ((n) % (p))) ? ((n) % (p)) : (i)))
 
-#define FORTRAN(nu,nl,pl,pc)        FORTRAN2(FORTRAN3(nu),FORTRAN3(nl),pl,pc)
+#define FORTRAN(nu,nl,pl,pc)        FORTRAN2(REPLACE(nu),REPLACE(nl),pl,pc)
 #define FORTRAN2(nu,nl,pl,pc)                    \
 void nu pl;                                      \
 void nl pl                                       \
 { nu pc; }                                       \
-void FORTRAN4(nl,_) pl	                         \
+void GLUE(nl,_) pl                               \
 { nu pc; }                                       \
-void FORTRAN4(nl,__) pl                          \
+void GLUE(nl,__) pl                              \
 { nu pc; }                                       \
 void nu pl
-#define FORTRAN3(s)                 s
-#define FORTRAN4(p,s)               p##s
+
+#define REPLACE(s)                  s
+#define GLUE(p,s)                   p##s
 
 #define STRINGIFY2(n)               #n
 #define STRINGIFY(n)                STRINGIFY2(n)
