@@ -1,4 +1,4 @@
-/* Copyright 2007-2010 ENSEIRB, INRIA & CNRS
+/* Copyright 2007-2010,2012,2015 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -38,7 +38,7 @@
 /**                Sebastien FOUCAULT (P0.0)               **/
 /**                Nicolas GICQUEL (P0.1)                  **/
 /**                Jerome LACOSTE (P0.1)                   **/
-/**                Cedric CHEVALIER (5.0)                  **/
+/**                Cedric CHEVALIER (v5.0)                 **/
 /**                                                        **/
 /**   FUNCTION   : These lines are the data declarations   **/
 /**                for the distributed source graph        **/
@@ -56,6 +56,8 @@
 /**                                 to   : 03 aug 2007     **/
 /**                # Version 5.1  : from : 11 nov 2007     **/
 /**                                 to   : 20 feb 2011     **/
+/**                # Version 6.0  : from : 30 aug 2012     **/
+/**                                 to   : 28 dec 2015     **/
 /**                                                        **/
 /************************************************************/
 
@@ -86,8 +88,8 @@
 #define DGRAPHFREEALL               (DGRAPHFREEPRIV | DGRAPHFREECOMM | DGRAPHFREETABS | DGRAPHFREEPSID  | DGRAPHFREEEDGEGST)
 #define DGRAPHCOMMPTOP              0x0100        /* Use point-to-point collective communication */
 
-#define DGRAPHBITSUSED              0x00FF        /* Significant bits for plain distributed graph routines               */
-#define DGRAPHBITSNOTUSED           0x0100        /* Value above which bits not used by plain distributed graph routines */
+#define DGRAPHBITSUSED              0x01FF        /* Significant bits for plain distributed graph routines               */
+#define DGRAPHBITSNOTUSED           0x0200        /* Value above which bits not used by plain distributed graph routines */
 
 /* Used in algorithms */
 
@@ -100,10 +102,11 @@
 
 /* The graph basic types, which must be signed. */
 
-#ifndef GNUMMAX                                   /* If graph.h not included */
-typedef INT                 Gnum;                 /* Vertex or edge number   */
-#define GNUMMAX                     (INTVALMAX)   /* Maximum Gnum value      */
-#define GNUMSTRING                  INTSTRING     /* String to printf a Gnum */
+#ifndef GNUMMAX                                   /* If graph.h not included    */
+typedef INT                 Gnum;                 /* Vertex or edge number      */
+#define GNUMMAX                     (INTVALMAX)   /* Maximum Gnum value         */
+#define GNUMMIN                     (-GNUMMAX - 1) /* Minimum signed Gnum value */
+#define GNUMSTRING                  INTSTRING     /* String to printf a Gnum    */
 #endif /* GNUMMAX */
 
 #define GNUM_MPI                    COMM_INT      /* MPI type for Gnum is MPI type for INT */
@@ -204,12 +207,12 @@ int                         dgraphCheck         (const Dgraph * const);
 int                         dgraphView          (const Dgraph * const, FILE * const);
 int                         dgraphGhst2         (Dgraph * const, const int);
 int                         dgraphBand          (Dgraph * restrict const, const Gnum, Gnum * restrict const, const GraphPart * restrict const, const Gnum, const Gnum, Gnum, Dgraph * restrict const, Gnum * restrict * const, GraphPart * restrict * const, Gnum * const, Gnum * const, Gnum * const);
-int                  dgraphBandColl      (Dgraph * restrict const, const Gnum, Gnum * restrict const, const Gnum, Gnum * restrict * restrict const, Gnum * restrict const, Gnum * restrict const, Gnum * restrict const);
-int                  dgraphBandPtop      (Dgraph * restrict const, const Gnum, Gnum * restrict const, const Gnum, Gnum * restrict * restrict const, Gnum * restrict const, Gnum * restrict const, Gnum * restrict const);
 
 int                         dgraphFold          (const Dgraph * restrict const, const int, Dgraph * restrict const, const void * restrict const, void ** restrict const, MPI_Datatype);
 int                         dgraphFold2         (const Dgraph * restrict const, const int, Dgraph * restrict const, MPI_Comm, const void * restrict const, void ** restrict const, MPI_Datatype);
 int                         dgraphFoldDup       (const Dgraph * restrict const, Dgraph * restrict const, void * restrict const, void ** restrict const, MPI_Datatype);
+int                         dgraphInduce2       (Dgraph * restrict const, Gnum (*) (Dgraph * restrict const, Dgraph * restrict const, const void * restrict const, Gnum * restrict const), const void * const, const Gnum, Gnum *, Dgraph * restrict const);
+
 int                         dgraphInduceList    (Dgraph * const, const Gnum, const Gnum * const, Dgraph * const);
 int                         dgraphInducePart    (Dgraph * const, const GraphPart * restrict const, const Gnum, const GraphPart, Dgraph * const);
 #ifdef GRAPH_H
