@@ -1,4 +1,4 @@
-/* Copyright 2004,2007-2010,2012,2014 Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2004,2007-2010,2012,2014,2018 Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -52,7 +52,7 @@
 /**                # Version 5.1  : from : 16 jun 2008     **/
 /**                                 to   : 15 aug 2010     **/
 /**                # Version 6.0  : from : 01 dec 2012     **/
-/**                                 to   : 12 nov 2014     **/
+/**                                 to   : 10 jul 2018     **/
 /**                                                        **/
 /************************************************************/
 
@@ -64,7 +64,7 @@
 
 #define CHARMAX                     2048          /* Maximum line size */
 
-#define SUBSMAX                     48            /* Maximum number of substitutions */
+#define SUBSMAX                     256           /* Maximum number of substitutions */
 
 #define C_FILENBR                   2             /* Number of files in list                */
 #define C_FILEARGNBR                2             /* Number of files which can be arguments */
@@ -78,6 +78,14 @@
 #define EXPAND(s)                   EXPANDTWO(s)
 #define EXPANDTWO(s)                #s
 
+#define subsFill(a,b)               {                            \
+                                      substab[subsnbr][0] = (a); \
+                                      substab[subsnbr][1] = (b); \
+                                      subsnbr ++;                \
+                                    }
+#define subsSuffix(a)               subsFill (a, subsSuffix2 (a, suffptr))
+
+#include <regex.h>
 #include "module.h"
 #include "common.h"
 #include "parser.h"
@@ -103,8 +111,8 @@
 
 static int                  C_fileNum = 0;        /* Number of file in arg list */
 static File                 C_fileTab[C_FILENBR] = { /* The file array          */
-                              { "r" },
-                              { "w" } };
+                              { FILEMODER },
+                              { FILEMODEW } };
 
 /******************************/
 /*                            */
@@ -112,19 +120,29 @@ static File                 C_fileTab[C_FILENBR] = { /* The file array
 /*                            */
 /******************************/
 
-void
-subsFill (
-char *                      substab[2],
-char *                      origptr,
+char *
+subsSize (
 int                         subsval)
 {
   char *              subsptr;
 
-  subsptr = malloc (32 * sizeof (char));
+  subsptr = malloc (16 * sizeof (char));
   sprintf (subsptr, "%d", (int) ((subsval + sizeof (double) - 1) / sizeof (double)));
 
-  substab[0] = origptr;
-  substab[1] = subsptr;
+  return (subsptr);
+}
+
+char *
+subsSuffix2 (
+char *                      nameptr,
+char *                      suffptr)
+{
+  char *              subsptr;
+
+  subsptr = malloc (128 * sizeof (char));
+  sprintf (subsptr, "%s%s", nameptr, suffptr);
+
+  return (subsptr);
 }
 
 /******************************/
@@ -138,9 +156,12 @@ main (
 int                         argc,
 char *                      argv[])
 {
+  regex_t             regedat;                    /* Regular expression for function names */
+  regmatch_t          matcdat;                    /* Matching structure                    */
   char                chartab[CHARMAX];
   char                chartmp[CHARMAX];
-  char *              substab[SUBSMAX][2];        /* Substitution array */
+  char *              substab[SUBSMAX][2];        /* Substitution array                    */
+  char *              suffptr;                    /* Pointer to suffix array               */
   int                 subsnbr;
   int                 i;
 
@@ -149,8 +170,10 @@ char *                      argv[])
     return (((argv[1][0] == '?') && argv[1][1] == '\0') ? 0 : 1);
   }
 
+  suffptr = "";                                   /* No suffix */
+
   for (i = 0; i < C_FILENBR; i ++)                /* Set default stream pointers */
-    C_fileTab[i].fileptr = (C_fileTab[i].modeptr[0] == 'r') ? stdin : stdout;
+    C_fileTab[i].fileptr = ((C_fileTab[i].flagval & FILEMODE) == FILEMODER) ? stdin : stdout;
   for (i = 1; i < argc; i ++) {                   /* Loop for all option codes */
     if ((argv[i][0] != '+') &&                    /* If found a file name      */
         ((argv[i][0] != '-') || (argv[i][1] == '\0'))) {
@@ -167,9 +190,13 @@ char *                      argv[])
         case 'h' :
           printf ("Usage is:\ndummysizes [<input pattern header file> [<output header file>]]\n");
           exit       (0);
+        case 'S' :
+        case 's' :
+          suffptr = &argv[i][2];
+          break;
         case 'V' :
           fprintf (stderr, "dummysizes, version " SCOTCH_VERSION_STRING "\n");
-          fprintf (stderr, "Copyright 2004,2007-2010 ENSEIRB, INRIA & CNRS, France\n");
+          fprintf (stderr, "Copyright 2004,2007-2010,2018 IPB, Universite de Bordeaux, INRIA & CNRS, France\n");
           fprintf (stderr, "This software is libre/free software under CeCILL-C -- see the user's manual for more information\n");
           return  (0);
         default :
@@ -182,59 +209,92 @@ char *                      argv[])
   for (i = 0; i < C_FILENBR; i ++) {              /* For all file names     */
     if ((C_fileTab[i].nameptr[0] != '-') ||       /* If not standard stream */
         (C_fileTab[i].nameptr[1] != '\0')) {
-      if ((C_fileTab[i].fileptr = fopen (C_fileTab[i].nameptr, C_fileTab[i].modeptr)) == NULL) { /* Open the file */
+      if ((C_fileTab[i].fileptr = fopen (C_fileTab[i].nameptr, ((C_fileTab[i].flagval & FILEMODE) == FILEMODER) ? "r" : "w")) == NULL) { /* Open the file */
           fprintf (stderr, "dummysizes: ERROR: main: cannot open file (%d)", i);
           exit    (1);
       }
     }
   }
 
+  subsnbr = 0;
 #ifdef SCOTCH_PTSCOTCH
-  substab[0][0] = "library_pt.h";
-  substab[0][1] = "ptscotch.h  ";
-  substab[1][0] = "library_pt_f.h";
-  substab[1][1] = "ptscotchf.h   ";
+  subsFill ("library_pt.h", "ptscotch.h  ");
+  subsFill ("library_pt_f.h", "ptscotchf.h   ");
+  subsSuffix ("PTSCOTCH_H");
 #else /* SCOTCH_PTSCOTCH */
-  substab[0][0] = "library.h";
-  substab[0][1] = "scotch.h ";
-  substab[1][0] = "library_f.h";
-  substab[1][1] = "scotchf.h  ";
+  subsFill ("library.h", "scotch.h ");
+  subsFill ("library_f.h", "scotchf.h  ");
+  subsSuffix ("SCOTCH_H");
 #endif /* SCOTCH_PTSCOTCH */
-  substab[2][0] = "DUMMYIDX";
-  substab[2][1] = EXPAND (IDX);
-  substab[3][0] = "DUMMYINT";
-  substab[3][1] = EXPAND (INT);
-  substab[4][0] = "DUMMYMAXINT";
-  substab[4][1] = EXPAND (INTVALMAX);
-  substab[5][0] = "DUMMYNUMSTRING";
-  substab[5][1] = "\"" GNUMSTRING "\"";
-  substab[6][0] = "DUMMYVERSION";
-  substab[6][1] = EXPAND (SCOTCH_VERSION);
-  substab[7][0] = "DUMMYRELEASE";
-  substab[7][1] = EXPAND (SCOTCH_RELEASE);
-  substab[8][0] = "DUMMYPATCHLEVEL";
-  substab[8][1] = EXPAND (SCOTCH_PATCHLEVEL);
-  subsnbr = 9;
-  subsFill (substab[subsnbr ++], "DUMMYSIZEARCH",          sizeof (Arch));
-  subsFill (substab[subsnbr ++], "DUMMYSIZEGEOM",          sizeof (Geom));
-  subsFill (substab[subsnbr ++], "DUMMYSIZEGRAPH",         sizeof (Graph));
-  subsFill (substab[subsnbr ++], "DUMMYSIZEMESH",          sizeof (Mesh));
-  subsFill (substab[subsnbr ++], "DUMMYSIZEMAP",           sizeof (LibMapping));
-  subsFill (substab[subsnbr ++], "DUMMYSIZEORDER",         sizeof (LibOrder));
-  subsFill (substab[subsnbr ++], "DUMMYSIZESTRAT",         sizeof (Strat *));
+  subsFill ("DUMMYIDX", EXPAND (IDX));
+  subsFill ("DUMMYINT", EXPAND (INT));
+  subsFill ("DUMMYMAXINT", EXPAND (INTVALMAX));
+  subsFill ("DUMMYNUMSTRING", "\"" GNUMSTRING "\"");
+  subsFill ("DUMMYVERSION", EXPAND (SCOTCH_VERSION_NUM));
+  subsFill ("DUMMYRELEASE", EXPAND (SCOTCH_RELEASE_NUM));
+  subsFill ("DUMMYPATCHLEVEL", EXPAND (SCOTCH_PATCHLEVEL_NUM));
+  subsFill ("DUMMYSIZEARCH", subsSize (sizeof (Arch)));
+  subsFill ("DUMMYSIZEGEOM", subsSize (sizeof (Geom)));
+  subsFill ("DUMMYSIZEGRAPH", subsSize (sizeof (Graph)));
+  subsFill ("DUMMYSIZEMESH",  subsSize (sizeof (Mesh)));
+  subsFill ("DUMMYSIZEMAP", subsSize (sizeof (LibMapping)));
+  subsFill ("DUMMYSIZEORDER", subsSize (sizeof (LibOrder)));
+  subsFill ("DUMMYSIZESTRAT", subsSize (sizeof (Strat *)));
 #ifdef SCOTCH_PTSCOTCH
-  subsFill (substab[subsnbr ++], "DUMMYSIZEDGRAPHHALOREQ", sizeof (DgraphHaloRequest)); /* TRICK: before DUMMYSIZEDGRAPH */
-  subsFill (substab[subsnbr ++], "DUMMYSIZEDGRAPH",        sizeof (Dgraph));
-  subsFill (substab[subsnbr ++], "DUMMYSIZEDMAP",          sizeof (LibDmapping));
-  subsFill (substab[subsnbr ++], "DUMMYSIZEDORDER",        sizeof (Dorder));
+  subsFill ("DUMMYSIZEDGRAPHHALOREQ", subsSize (sizeof (DgraphHaloRequest))); /* TRICK: before DUMMYSIZEDGRAPH */
+  subsFill ("DUMMYSIZEDGRAPH", subsSize (sizeof (Dgraph)));
+  subsFill ("DUMMYSIZEDMAP", subsSize (sizeof (LibDmapping)));
+  subsFill ("DUMMYSIZEDORDER", subsSize (sizeof (Dorder)));
 #else /* SCOTCH_PTSCOTCH */
-  subsFill (substab[subsnbr ++], "DUMMYSIZEDGRAPHHALOREQ", 1); /* TRICK: before DUMMYSIZEDGRAPH */
-  subsFill (substab[subsnbr ++], "DUMMYSIZEDGRAPH",        1);
-  subsFill (substab[subsnbr ++], "DUMMYSIZEDMAP",          1);
-  subsFill (substab[subsnbr ++], "DUMMYSIZEDORDER",        1);
+  subsFill ("DUMMYSIZEDGRAPHHALOREQ", "1");       /* TRICK: before DUMMYSIZEDGRAPH */
+  subsFill ("DUMMYSIZEDGRAPH", "1");
+  subsFill ("DUMMYSIZEDMAP", "1");
+  subsFill ("DUMMYSIZEDORDER", "1");
 #endif /* SCOTCH_PTSCOTCH */
+  subsFill ("##SCOTCH_NAME_SUFFIX", suffptr);
+  if (suffptr[0] != '\0') {                       /* If suffix provided */
+#ifdef SCOTCH_RENAME_ALL
+    subsSuffix ("SCOTCH_VERSION");
+    subsSuffix ("SCOTCH_RELEASE");
+    subsSuffix ("SCOTCH_PATCHLEVEL");
+    subsSuffix ("SCOTCH_COARSENNONE");
+    subsSuffix ("SCOTCH_COARSENFOLD");
+    subsSuffix ("SCOTCH_COARSENFOLDDUP");
+    subsSuffix ("SCOTCH_COARSENNOMERGE");
+    subsSuffix ("SCOTCH_STRATDEFAULT");
+    subsSuffix ("SCOTCH_STRATQUALITY");
+    subsSuffix ("SCOTCH_STRATSPEED");
+    subsSuffix ("SCOTCH_STRATBALANCE");
+    subsSuffix ("SCOTCH_STRATSAFETY");
+    subsSuffix ("SCOTCH_STRATSCALABILITY");
+    subsSuffix ("SCOTCH_STRATRECURSIVE");
+    subsSuffix ("SCOTCH_STRATREMAP");
+    subsSuffix ("SCOTCH_STRATLEVELMAX");
+    subsSuffix ("SCOTCH_STRATLEVELMIN");
+    subsSuffix ("SCOTCH_STRATLEAFSIMPLE");
+    subsSuffix ("SCOTCH_STRATSEPASIMPLE");
+#endif /* SCOTCH_RENAME_ALL */
+#ifdef SCOTCH_PTSCOTCH
+#else /* SCOTCH_PTSCOTCH */
+    subsSuffix ("SCOTCH_Idx");
+    subsSuffix ("SCOTCH_Num");
+    subsSuffix ("SCOTCH_NUMMAX");
+    subsSuffix ("SCOTCH_NUMSTRING");
+    subsSuffix ("SCOTCH_Arch");
+    subsSuffix ("SCOTCH_Geom");
+    subsSuffix ("SCOTCH_Graph");
+    subsSuffix ("SCOTCH_Mesh");
+    subsSuffix ("SCOTCH_Mapping");
+    subsSuffix ("SCOTCH_Ordering");
+    subsSuffix ("SCOTCH_Strat");
+#endif /* SCOTCH_PTSCOTCH */
+    if (regcomp (&regedat, " SCOTCH_[a-z][0-9a-zA-Z_]*", 0) != 0) {
+      fprintf (stderr, "dummysizes: ERROR: cannot compile regular expression\n");
+      exit    (1);
+    }
+  }
 
-  while (fgets (chartab, CHARMAX, C_filepntrhedinp) != NULL) { /* Infinite loop on file lines */
+  while (fgets (chartab, CHARMAX, C_filepntrhedinp) != NULL) { /* Loop on file lines */
     int                 charnbr;
     int                 subsnum;
 
@@ -247,22 +307,34 @@ char *                      argv[])
     for (subsnum = 0; subsnum < subsnbr; subsnum ++) { /* Perform substitutions */
       char *              charptr;                /* Place where token found    */
 
-      while ((charptr = strstr (chartab, substab[subsnum][0])) != NULL) { /* As long as substitution can be performed */
-        int                 charnbr;
-        int                 charnum;
+      charptr = chartab;                          /* Start from beginning of string                      */
+      while ((charptr = strstr (charptr, substab[subsnum][0])) != NULL) { /* As long as a matching found */
+        int                 subslen;
 
-        charnum = charptr - chartab;              /* Position where token found */
-        charnbr = strlen (substab[subsnum][0]);   /* Length of token            */
+        subslen = strlen (substab[subsnum][0]);
+        if (isalnum (charptr[subslen]) || (charptr[subslen] == '_')) { /* If next character is part of a longer identifier */
+          charptr += subslen + 1;                 /* Skip substitution */
+          continue;
+        }
 
-        strcpy (chartmp, charptr + charnbr);      /* Save end of line */
-
-        sprintf (charptr, "%s%s", substab[subsnum][1], chartmp); /* Replace end of line with substituted token */
+        strcpy (chartmp, charptr + subslen);      /* Save remaining of line                                          */
+        sprintf (charptr, "%s%s", substab[subsnum][1], chartmp); /* Replace remaining of line with substituted token */
+        charptr += strlen (substab[subsnum][1]);  /* Restart search from end of substituted token                    */
+      }
+    }
+    if (suffptr[0] != '\0') {                     /* If suffix provided                     */
+      if (regexec (&regedat, chartab, 1, &matcdat, 0) == 0) { /* If matched a function name */
+        strcpy (chartmp, chartab + matcdat.rm_eo); /* Save remaining of line                */
+        sprintf (chartab + matcdat.rm_eo, "%s%s", suffptr, chartmp); /* Add suffix to name  */
       }
     }
 
     fputs (chartab, C_filepntrhedout);            /* Output possibly updated line */
   }
 
+  if (suffptr[0] != '\0')                         /* If suffix provided      */
+    regfree (&regedat);                           /* Free regular expression */
+
 #ifdef SCOTCH_DEBUG_MAIN1
   for (i = 0; i < C_FILENBR; i ++) {              /* For all file names     */
     if ((C_fileTab[i].name[0] != '-') ||          /* If not standard stream */
