@@ -1,4 +1,4 @@
-/* Copyright 2004,2007 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007,2010,2016 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -54,6 +54,10 @@
 /**                                 to     21 jan 2004     **/
 /**                # Version 5.0  : from : 13 dec 2006     **/
 /**                                 to     10 sep 2007     **/
+/**                # Version 5.1  : from : 11 aug 2010     **/
+/**                                 to     11 aug 2010     **/
+/**                # Version 6.0  : from : 03 aug 2016     **/
+/**                                 to     03 aug 2016     **/
 /**                                                        **/
 /************************************************************/
 
@@ -90,11 +94,11 @@ const GraphFlag             flagval)              /* Graph loading flags
 {
   Gnum                edgenum;                    /* Number of edges really allocated */
   Gnum                edgennd;
-  Gnum                vlblnbr;                    /* = vertnbr if vertex labels       */
+  Gnum                vlblsiz;                    /* = vertnbr if vertex labels       */
   Gnum                vlblmax;                    /* Maximum vertex label number      */
-  Gnum                velonbr;                    /* = vertnbr if vertex loads wanted */
+  Gnum                velosiz;                    /* = vertnbr if vertex loads wanted */
   Gnum                velosum;                    /* Sum of vertex loads              */
-  Gnum                edlonbr;                    /* = edgenbr if edge loads wanted   */
+  Gnum                edlosiz;                    /* = edgenbr if edge loads wanted   */
   Gnum                edlosum;                    /* Sum of edge loads                */
   Gnum                edgeval;                    /* Value where to read edge end     */
   Gnum                baseadj;
@@ -134,24 +138,24 @@ const GraphFlag             flagval)              /* Graph loading flags
     grafptr->baseval = baseadj;                   /* Set graph base as file base */
     baseadj          = 0;                         /* No base adjustment needed   */
   }
-  else {                                          /* If set graph base     */
-    grafptr->baseval = baseval;                   /* Set wanted graph base */
-    baseadj          = baseval - baseadj;         /* Update base adjust    */
+  else {                                          /* If prescribed graph base */
+    grafptr->baseval = baseval;                   /* Set wanted graph base    */
+    baseadj          = baseval - baseadj;         /* Update base adjust       */
   }
   if (proptab[0] != 0)                            /* If vertex labels, no base adjust */
     baseadj = 0;
 
-  velonbr = ((proptab[2] != 0) && ((flagval & GRAPHIONOLOADVERT) == 0)) ? grafptr->vertnbr : 0;
-  vlblnbr = (proptab[0] != 0) ? grafptr->vertnbr : 0;
-  edlonbr = ((proptab[1] != 0) && ((flagval & GRAPHIONOLOADEDGE) == 0)) ? grafptr->edgenbr : 0;
+  velosiz = ((proptab[2] != 0) && ((flagval & GRAPHIONOLOADVERT) == 0)) ? grafptr->vertnbr : 0;
+  vlblsiz = (proptab[0] != 0) ? grafptr->vertnbr : 0;
+  edlosiz = ((proptab[1] != 0) && ((flagval & GRAPHIONOLOADEDGE) == 0)) ? grafptr->edgenbr : 0;
 
   if ((memAllocGroup ((void **) (void *)
                       &grafptr->verttax, (size_t) ((grafptr->vertnbr + 1) * sizeof (Gnum)),
-                      &grafptr->velotax, (size_t) (velonbr                * sizeof (Gnum)),
-                      &grafptr->vlbltax, (size_t) (vlblnbr                * sizeof (Gnum)), NULL) == NULL) ||
+                      &grafptr->velotax, (size_t) (velosiz                * sizeof (Gnum)),
+                      &grafptr->vlbltax, (size_t) (vlblsiz                * sizeof (Gnum)), NULL) == NULL) ||
       (memAllocGroup ((void **) (void *)
                       &grafptr->edgetax, (size_t) (grafptr->edgenbr       * sizeof (Gnum)),
-                      &grafptr->edlotax, (size_t) (edlonbr                * sizeof (Gnum)), NULL) == NULL)) {
+                      &grafptr->edlotax, (size_t) (edlosiz                * sizeof (Gnum)), NULL) == NULL)) {
     if (grafptr->verttax != NULL)
       memFree (grafptr->verttax);
     errorPrint ("graphLoad: out of memory");
@@ -161,10 +165,10 @@ const GraphFlag             flagval)              /* Graph loading flags
   grafptr->vertnnd  = grafptr->vertnbr + grafptr->baseval;
   grafptr->verttax -= grafptr->baseval;
   grafptr->vendtax  = grafptr->verttax + 1;       /* Use compact vertex array */
-  grafptr->velotax  = (velonbr != 0) ? (grafptr->velotax - grafptr->baseval) : NULL;
-  grafptr->vlbltax  = (vlblnbr != 0) ? (grafptr->vlbltax - grafptr->baseval) : NULL;
+  grafptr->velotax  = (velosiz != 0) ? (grafptr->velotax - grafptr->baseval) : NULL;
+  grafptr->vlbltax  = (vlblsiz != 0) ? (grafptr->vlbltax - grafptr->baseval) : NULL;
   grafptr->edgetax -= grafptr->baseval;
-  grafptr->edlotax  = (edlonbr != 0) ? (grafptr->edlotax - grafptr->baseval) : NULL;
+  grafptr->edlotax  = (edlosiz != 0) ? (grafptr->edlotax - grafptr->baseval) : NULL;
 
   vlblmax = grafptr->vertnnd - 1;                 /* No vertex labels known */
   velosum = (grafptr->velotax == NULL) ? grafptr->vertnbr : 0;
@@ -190,7 +194,7 @@ const GraphFlag             flagval)              /* Graph loading flags
     if (proptab[2] != 0) {                        /* If must read vertex load        */
       Gnum                veloval;                /* Value where to read vertex load */
 
-      if (intLoad (stream, &veloval) != 1) {      /* Read vertex load data    */
+      if (intLoad (stream, &veloval) != 1) {      /* Read vertex load data */
         errorPrint ("graphLoad: bad input (4)");
         graphFree  (grafptr);
         return     (1);
@@ -338,10 +342,10 @@ FILE * const                stream)
   propstr[2] = (grafptr->velotax != NULL) ? '1' : '0';
   propstr[3] = '\0';
 
-  if (fprintf (stream, "0\n%ld\t%ld\n%ld\t%3s\n", /* Write file header */
-               (long) grafptr->vertnbr,
-               (long) grafptr->edgenbr,
-               (long) grafptr->baseval,
+  if (fprintf (stream, "0\n" GNUMSTRING "\t" GNUMSTRING "\n" GNUMSTRING "\t%3s\n", /* Write file header */
+               (Gnum) grafptr->vertnbr,
+               (Gnum) grafptr->edgenbr,
+               (Gnum) grafptr->baseval,
                propstr) == EOF) {
     errorPrint ("graphSave: bad output (1)");
     return     (1);
@@ -352,11 +356,11 @@ FILE * const                stream)
     Gnum                edgenum;
 
     if (grafptr->vlbltax != NULL)                 /* Write vertex label if necessary */
-      o  = (fprintf (stream, "%ld\t", (long) grafptr->vlbltax[vertnum]) == EOF);
+      o  = (fprintf (stream, GNUMSTRING "\t", (Gnum) grafptr->vlbltax[vertnum]) == EOF);
     if (grafptr->velotax != NULL)                 /* Write vertex load if necessary */
-      o |= (fprintf (stream, "%ld\t", (long) grafptr->velotax[vertnum]) == EOF);
+      o |= (fprintf (stream, GNUMSTRING "\t", (Gnum) grafptr->velotax[vertnum]) == EOF);
 
-    o |= (fprintf (stream, "%ld", (long) (grafptr->vendtax[vertnum] - grafptr->verttax[vertnum])) == EOF); /* Write vertex degree */
+    o |= (fprintf (stream, GNUMSTRING, (Gnum) (grafptr->vendtax[vertnum] - grafptr->verttax[vertnum])) == EOF); /* Write vertex degree */
 
     for (edgenum = grafptr->verttax[vertnum];
          (edgenum < grafptr->vendtax[vertnum]) && (o == 0); edgenum ++) {
@@ -364,9 +368,9 @@ FILE * const                stream)
 
       o |= (putc ('\t', stream) == EOF);
       if (grafptr->edlotax != NULL)               /* Write edge load if necessary */
-        o |= (fprintf (stream, "%ld ", (long) grafptr->edlotax[edgenum]) == EOF);
+        o |= (fprintf (stream, GNUMSTRING "\t", (Gnum) grafptr->edlotax[edgenum]) == EOF);
       vertend = grafptr->edgetax[edgenum];
-      o |= (fprintf (stream, "%ld", (long) ((grafptr->vlbltax != NULL) ? grafptr->vlbltax[vertend] : vertend)) == EOF); /* Write edge end */
+      o |= (fprintf (stream, GNUMSTRING, (Gnum) ((grafptr->vlbltax != NULL) ? grafptr->vlbltax[vertend] : vertend)) == EOF); /* Write edge end */
     }
     o |= (putc ('\n', stream) == EOF);
   }
