@@ -1,4 +1,4 @@
-/* Copyright 2004,2007 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007,2008,2010,2016,2018 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -47,6 +47,10 @@
 /**                                 to     21 mar 2005     **/
 /**                # Version 5.0  : from : 06 jun 2007     **/
 /**                                 to     31 aug 2007     **/
+/**                # Version 5.1  : from : 09 nov 2008     **/
+/**                                 to     27 apr 2010     **/
+/**                # Version 6.0  : from : 04 aug 2016     **/
+/**                                 to     31 may 2018     **/
 /**                                                        **/
 /************************************************************/
 
@@ -75,7 +79,7 @@ Graph * restrict const      grafptr,              /* Graph to load    */
 Geom * restrict const       geomptr,              /* Geometry to load */
 FILE * const                filesrcptr,           /* Topological data */
 FILE * const                filegeoptr,           /* No use           */
-const char * const          dataptr)              /* No use           */
+const char * const          dataptr)              /* Tag value        */
 {
   Gnum                          habmattag;        /* Matrix tag number in file       */
   Gnum                          habmatnum;        /* Current matrix number           */
@@ -83,7 +87,6 @@ const char * const          dataptr)              /* No use           */
   char                          habmattype[3];    /* Matrix type                     */
   Gnum                          habcrdnbr;        /* Total number of data lines      */
   Gnum                          habrhsnbr;        /* Number of right hand side lines */
-  Gnum                          habrownbr;        /* Number of rows                  */
   GraphGeomHaboLine             habcolfmt;        /* Format of column line           */
   int                           habvalnum;        /* Number of value in line         */
   Gnum                          habcolnbr;        /* Number of columns               */
@@ -102,9 +105,11 @@ const char * const          dataptr)              /* No use           */
   Gnum                          degrmax;          /* Maximum degree                  */
   int                           c;
 
-  if (((habmattag = (Gnum) atol (dataptr)) == 0) && /* Get tag value */
-      (dataptr[0] != '0') && (dataptr[0] != '\0')) {
-    errorPrint ("graphGeomLoadHabo: bad input (1)");
+  if ((dataptr != NULL)                          && /* If tag value provided */
+      (dataptr[0] != '\0')                       &&
+      ((habmattag = (Gnum) atol (dataptr)) == 0) && /* Get tag value */
+      (dataptr[0] != '0')) {
+    errorPrint ("graphGeomLoadHabo: invalid parameter");
     return     (1);
   }
 
@@ -117,7 +122,7 @@ const char * const          dataptr)              /* No use           */
         (fgets (habmatbuf[1], 83, filesrcptr) == NULL) ||
         (fgets (habmatbuf[2], 83, filesrcptr) == NULL) ||
         (fgets (habmatbuf[3], 83, filesrcptr) == NULL)) {
-      errorPrint ("graphGeomLoadHabo: bad input (2)");
+      errorPrint ("graphGeomLoadHabo: bad input (1)");
       return     (1);
     }
     habmatbuf[1][70] = '\0';                      /* Extract header values */
@@ -131,24 +136,26 @@ const char * const          dataptr)              /* No use           */
     habnzrnbr = (Gnum) atol (&habmatbuf[2][43]);
     habmatbuf[2][42] = '\0';
     habcolnbr = (Gnum) atol (&habmatbuf[2][29]);
+#if 0                                             /* Number of rows not used since only square matrices considered */
     habmatbuf[2][28] = '\0';
     habrownbr = (Gnum) atol (&habmatbuf[2][14]);
+#endif
 
     habmatbuf[3][32] = '\0';
     if (graphGeomLoadHaboFormat (&habnzrfmt, &habmatbuf[3][16]) != 0) {
-      errorPrint ("graphGeomLoadHabo: bad input (3)");
+      errorPrint ("graphGeomLoadHabo: bad input (2)");
       return     (1);
     }
     habmatbuf[3][16] = '\0';
     if (graphGeomLoadHaboFormat (&habcolfmt, &habmatbuf[3][0]) != 0) {
-      errorPrint ("graphGeomLoadHabo: bad input (4)");
+      errorPrint ("graphGeomLoadHabo: bad input (3)");
       return     (1);
     }
 
     if (habrhsnbr != 0) {
       while ((c = getc (filesrcptr)) != '\n'){    /* Skip RHS format line */
         if (c == EOF) {
-          errorPrint ("graphGeomLoadHabo: bad input (5)");
+          errorPrint ("graphGeomLoadHabo: bad input (4)");
           return     (1);
         }
       }
@@ -158,7 +165,7 @@ const char * const          dataptr)              /* No use           */
       while (habcrdnbr -- > 0) {                 /* Skip all of file lines  */
         while ((c = getc (filesrcptr)) != '\n') { /* Skip line              */
           if (c == EOF) {
-            errorPrint ("graphGeomLoadHabo: bad input (6)");
+            errorPrint ("graphGeomLoadHabo: bad input (5)");
             return     (1);
           }
         }
@@ -176,10 +183,10 @@ const char * const          dataptr)              /* No use           */
   }
 
   if (((grafptr->verttax = (Gnum *) memAlloc ((habcolnbr + 1) * sizeof (Gnum))) == NULL) ||
-      ((grafptr->edgetax = (Gnum *) memAllocGroup ((void **) (void *)
-                                                   &grafptr->edgetax, (size_t) (habnzrnbr * 2   * sizeof (Gnum)),
-                                                   &habcoltab,        (size_t) ((habcolnbr + 1) * sizeof (Gnum)),
-                                                   &habnzrtab,        (size_t) (habnzrnbr       * sizeof (Gnum)), NULL)) == NULL)) {
+      (memAllocGroup ((void **) (void *)
+                      &grafptr->edgetax, (size_t) (habnzrnbr * 2   * sizeof (Gnum)),
+                      &habcoltab,        (size_t) ((habcolnbr + 1) * sizeof (Gnum)),
+                      &habnzrtab,        (size_t) (habnzrnbr       * sizeof (Gnum)), NULL) == NULL)) {
     errorPrint ("graphGeomLoadHabo: out of memory (1)");
     if (grafptr->verttax != NULL) {
       memFree (grafptr->verttax);
@@ -187,7 +194,7 @@ const char * const          dataptr)              /* No use           */
     }
     return (1);
   }
-  grafptr->flagval = GRAPHFREETABS;               /* Totally new graph structure       */
+  grafptr->flagval = GRAPHFREETABS | GRAPHVERTGROUP | GRAPHEDGEGROUP;
   grafptr->baseval = 1;                           /* Harwell-Boeing graphs have base 1 */
   grafptr->vertnbr = (Gnum) habcolnbr;
   grafptr->vertnnd = grafptr->vertnbr + 1;
@@ -195,6 +202,10 @@ const char * const          dataptr)              /* No use           */
   grafptr->vendtax = grafptr->verttax;            /* Use compact representation for array based at 1     */
   grafptr->verttax --;                            /* Base verttab array at 1, with vendtab = verttab + 1 */
   grafptr->edgetax --;
+  grafptr->velotax = NULL;
+  grafptr->vnumtax = NULL;
+  grafptr->vlbltax = NULL;
+  grafptr->edlotax = NULL;
 
   ungetc ('\n', filesrcptr);                      /* Create fake previous line     */
   for (habcolnum = 0, habvalnum = habcolfmt.datanbr; /* Eat up fake previous line  */
@@ -220,14 +231,14 @@ const char * const          dataptr)              /* No use           */
         habcolval = habcolval * 10 + c - '0';
     }
     if (c == EOF) {
-      errorPrint ("graphGeomLoadHabo: bad input (7)");
+      errorPrint ("graphGeomLoadHabo: bad input (6)");
       graphFree  (grafptr);
       return     (1);
     }
     habcoltab[habcolnum] = habcolval;
   }
   if (habcoltab[habcolnbr] != (Gnum) habnzrnbr + 1) {
-    errorPrint ("graphGeomLoadHabo: bad input (8)");
+    errorPrint ("graphGeomLoadHabo: bad input (7)");
     graphFree  (grafptr);
     return     (1);
   }
@@ -257,7 +268,7 @@ const char * const          dataptr)              /* No use           */
           habnzrval = habnzrval * 10 + c - '0';
       }
       if (c == EOF) {
-        errorPrint ("graphGeomLoadHabo: bad input (9)");
+        errorPrint ("graphGeomLoadHabo: bad input (8)");
         graphFree  (grafptr);
         return     (1);
       }
@@ -354,7 +365,7 @@ static
 int
 graphGeomLoadHaboFormat (
 GraphGeomHaboLine * restrict const  lineptr,      /* Line format to fill */
-const char * restrict const         dataptr)      /* Format string       */
+const char * const                  dataptr)      /* Format string       */
 {
   const char * restrict   charptr;
   int                     number;                 /* Number to read */
