@@ -1,4 +1,4 @@
-/* Copyright 2007 ENSEIRB, INRIA & CNRS
+/* Copyright 2007,2010,2018 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -41,6 +41,10 @@
 /**                                                        **/
 /**   DATES      : # Version 5.0  : from : 15 apr 2006     **/
 /**                                 to     16 jun 2007     **/
+/**                # Version 5.1  : from : 04 nov 2010     **/
+/**                                 to     04 nov 2010     **/
+/**                # Version 6.0  : from : 07 jun 2018     **/
+/**                                 to     07 jun 2018     **/
 /**                                                        **/
 /************************************************************/
 
@@ -82,7 +86,7 @@
 typedef struct Hdgraph_ {
   Dgraph                    s;                    /*+ Source distributed graph                       +*/
   Gnum                      vhallocnbr;           /*+ Local number of halo end vertices              +*/
-  Gnum * restrict           vhndloctax;           /*+ End vertex array including halo vertex indices +*/
+  Gnum *                    vhndloctax;           /*+ End vertex array including halo vertex indices +*/
   Gnum                      ehallocnbr;           /*+ Local number of halo edges                     +*/
   Gnum                      levlnum;              /*+ Nested dissection level                        +*/
 } Hdgraph;
@@ -91,10 +95,6 @@ typedef struct Hdgraph_ {
 **  The function prototypes.
 */
 
-#ifndef HDGRAPH
-#define static
-#endif
-
 int                         hdgraphInit         (Hdgraph * const);
 void                        hdgraphExit         (Hdgraph * const);
 void                        hdgraphFree         (Hdgraph * const);
@@ -105,5 +105,3 @@ int                         hdgraphCheck        (const Hdgraph *);
 int                         hdgraphGather       (Hdgraph *, Hgraph *);
 #endif /* HGRAPH_H */
 int                         hdgraphInduceList   (Hdgraph * restrict const, const Gnum, const Gnum * restrict const, Hdgraph * restrict const);
-
-#undef static
