@@ -1,4 +1,4 @@
-/* Copyright 2008-2011 ENSEIRB, INRIA & CNRS
+/* Copyright 2008-2012,2014,2018 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -34,6 +34,7 @@
 /**   NAME       : kdgraph_map_rb_part.c                   **/
 /**                                                        **/
 /**   AUTHOR     : Francois PELLEGRINI                     **/
+/**                Sebastien FOURESTIER (v6.0)             **/
 /**                                                        **/
 /**   FUNCTION   : This module performs the Dual Recursive **/
 /**                Bipartitioning mapping algorithm        **/ 
@@ -45,6 +46,8 @@
 /**                                                        **/
 /**   DATES      : # Version 5.1  : from : 21 jun 2008     **/
 /**                                 to     31 aug 2011     **/
+/**                # Version 6.0  : from : 03 mar 2011     **/
+/**                                 to     03 jun 2018     **/
 /**                                                        **/
 /************************************************************/
 
@@ -101,18 +104,13 @@ const KdgraphMapRbPartData * restrict const dataptr)
   DmappingFrag * restrict   fragptr;
 
   cgrfptr = &grafptr->data.cgrfdat;
-  if (mapInit2 (&kgrfdat.m, cgrfptr->baseval, cgrfptr->vertnbr, &mappptr->archdat, &grafptr->domnorg) != 0) {
-    errorPrint ("kdgraphMapRbPartSequ: cannot initialize centralized mapping");
-    return     (1);
-  }
-
-  if (kgraphInit (&kgrfdat, cgrfptr, &kgrfdat.m) != 0) {
+  if (kgraphInit (&kgrfdat, cgrfptr, &mappptr->archdat, &grafptr->domnorg, 0, NULL, NULL, 1, 1, NULL) != 0) {
     errorPrint ("kdgraphMapRbPartSequ: cannot initialize centralized graph");
     return     (1);
   }
-  kgrfdat.s.flagval   = cgrfptr->flagval;         /* Free sequential graph along with mapping data    */
-  kgrfdat.s.vnumtax   = NULL;                     /* Remove index array if any                        */
-  kgrfdat.comploadrat = dataptr->comploadrat;     /* Use ideal load of full graph and not of subgraph */
+  kgrfdat.s.flagval   = (kgrfdat.s.flagval & ~GRAPHBITSUSED) | cgrfptr->flagval; /* Free sequential graph along with mapping data */
+  kgrfdat.s.vnumtax   = NULL;                     /* Remove index array if any                                                    */
+  kgrfdat.comploadrat = dataptr->comploadrat;     /* Use ideal load of full graph and not of subgraph                             */
 
   if (kgraphMapSt (&kgrfdat, dataptr->paraptr->stratseq) != 0) { /* Compute sequential mapping */
     kgraphExit (&kgrfdat);
@@ -129,7 +127,7 @@ const KdgraphMapRbPartData * restrict const dataptr)
   }
 
   fragptr->vertnbr = cgrfptr->vertnbr;
-  fragptr->parttab = kgrfdat.m.parttax + kgrfdat.m.baseval;
+  fragptr->parttab = kgrfdat.m.parttax + kgrfdat.s.baseval;
   fragptr->domnnbr = kgrfdat.m.domnnbr;
   fragptr->domntab = kgrfdat.m.domntab;
   kgrfdat.m.parttax = NULL;                       /* Keep sequential mapping arrays for distributed mapping fragment */
@@ -178,6 +176,7 @@ void * const                    dataptr)          /* Pointer to thread data */
     return ((void *) (intptr_t) kdgraphMapRbAddPart (fldthrdptr->orggrafptr, fldthrdptr->mappptr, fldthrdptr->inddomnptr, fldthrdptr->indvertnbr,
                                                      fldthrdptr->indparttax + fldthrdptr->orggrafptr->baseval, fldthrdptr->indpartval));
 
+  dgraphInit (&indgrafdat, fldthrdptr->orggrafptr->proccomm); /* Re-use communicator of original graph                          */
   if (dgraphInducePart (fldthrdptr->orggrafptr, fldthrdptr->indparttax, /* Compute unfinished induced subgraph on all processes */
                         fldthrdptr->indvertnbr, fldthrdptr->indpartval, &indgrafdat) != 0)
     return ((void *) 1);
@@ -185,6 +184,7 @@ void * const                    dataptr)          /* Pointer to thread data */
   if (fldthrdptr->fldprocnbr > 1) {               /* If subpart has several processes, fold a distributed graph                     */
     o = (void *) (intptr_t) dgraphFold2 (&indgrafdat, fldthrdptr->fldpartval, /* Fold temporary induced subgraph from all processes */
                                          &fldgrafptr->data.dgrfdat, fldthrdptr->fldproccomm, NULL, NULL, MPI_INT);
+    fldgrafptr->data.dgrfdat.flagval |= DGRAPHFREECOMM; /* Split communicator has to be freed */
   }
   else {                                          /* Create a centralized graph */
     Graph * restrict      fldcgrfptr;
@@ -202,45 +202,44 @@ int
 kdgraphMapRbPartFold (
 Bdgraph * restrict const                actgrafptr,
 Dmapping * restrict const               mappptr,
-const ArchDom * restrict const          domsubtab,
+const ArchDom * restrict const          domnsubtab,
 KdgraphMapRbPartGraph * restrict const  fldgrafptr)
 {
   KdgraphMapRbPartThread  fldthrdtab[2];
-  int                     fldprocnbr;             /* Number of processes in part of this process  */
   int                     fldprocnbr0;            /* Number of processes in first part            */
   int                     fldprocnum;
   int                     fldproccol;
   int                     fldpartval;
   Gnum                    indvertlocmax;          /* Local number of vertices in biggest subgraph */
-  int                     indconttab[2];          /* Array of subjob continuation flags           */
-  int                     indpartmax;             /* Induced part having most vertices            */
+  Gnum                    indflagtab[2];          /* Array of subjob continuation flags           */
+  GraphPart               indpartmax;             /* Induced part having most vertices            */
 #ifdef SCOTCH_PTHREAD
   Dgraph                  orggrafdat;             /* Structure for copying graph fields except communicator */
   pthread_t               thrdval;                /* Data of second thread                                  */
 #endif /* SCOTCH_PTHREAD */
   int                       o;
 
-  indconttab[0] =                                 /* Assume both jobs will not continue */
-  indconttab[1] = 0;
+  indflagtab[0] =                                 /* Assume both jobs will not continue */
+  indflagtab[1] = 0;
   if ((actgrafptr->compglbsize0 != 0) &&          /* If graph has been bipartitioned */
       (actgrafptr->compglbsize0 != actgrafptr->s.vertglbnbr)) {
     if (archVar (&mappptr->archdat)) {            /* If architecture is variable-sized    */
       if (actgrafptr->compglbsize0 > 1)           /* If graph is not single vertex, go on */
-        indconttab[0] = 1;
+        indflagtab[0] = ~0;                       /* All bits set to 1                    */
       if ((actgrafptr->s.vertglbnbr - actgrafptr->compglbsize0) > 1)
-        indconttab[1] = 1;
+        indflagtab[1] = ~0;
     }
-    else {                                        /* Architecture is not variable-sized         */
-      if (archDomSize (&mappptr->archdat, &domsubtab[0]) > 1) /* Stop when target is one vertex */
-        indconttab[0] = 1;
-      if (archDomSize (&mappptr->archdat, &domsubtab[1]) > 1)
-        indconttab[1] = 1;
+    else {                                        /* Architecture is not variable-sized          */
+      if (archDomSize (&mappptr->archdat, &domnsubtab[0]) > 1) /* Stop when target is one vertex */
+        indflagtab[0] = ~0;
+      if (archDomSize (&mappptr->archdat, &domnsubtab[1]) > 1)
+        indflagtab[1] = ~0;
     }
   }
 
-  if ((indconttab[0] + indconttab[1]) == 0) {     /* If both subjobs stop    */
+  if ((indflagtab[0] | indflagtab[1]) == 0) {     /* If both subjobs stop    */
     fldgrafptr->procnbr = 0;                      /* Nothing to do on return */
-    return (kdgraphMapRbAddBoth (&actgrafptr->s, mappptr, domsubtab, actgrafptr->partgsttax + actgrafptr->s.baseval)); /* Map both subdomains in the same time */
+    return (kdgraphMapRbAddBoth (&actgrafptr->s, mappptr, domnsubtab, actgrafptr->partgsttax + actgrafptr->s.baseval)); /* Map both subdomains in the same time */
   }
 
   if ((2 * actgrafptr->compglbsize0) >= actgrafptr->s.vertglbnbr) { /* Get part of largest subgraph */
@@ -255,32 +254,30 @@ KdgraphMapRbPartGraph * restrict const  fldgrafptr)
 
   fldthrdtab[0].mappptr     = mappptr;            /* Load data to pass to the subgraph building routines */
   fldthrdtab[0].orggrafptr  = &actgrafptr->s;
-  fldthrdtab[0].inddomnptr  = &domsubtab[indpartmax];
+  fldthrdtab[0].inddomnptr  = &domnsubtab[indpartmax];
   fldthrdtab[0].indvertnbr  = indvertlocmax;
   fldthrdtab[0].indpartval  = indpartmax;
   fldthrdtab[0].indparttax  = actgrafptr->partgsttax;
   fldthrdtab[0].fldgrafptr  = fldgrafptr;
   fldthrdtab[0].fldpartval  = 0;
-  fldthrdtab[0].fldprocnbr  = indconttab[indpartmax] * fldprocnbr0; /* Stop if domain limited to one vertex */
+  fldthrdtab[0].fldprocnbr  = indflagtab[indpartmax] & fldprocnbr0; /* Stop if domain limited to one vertex */
   fldthrdtab[1].mappptr     = mappptr;
   fldthrdtab[1].orggrafptr  = &actgrafptr->s;     /* Assume jobs won't be run concurrently */
-  fldthrdtab[1].inddomnptr  = &domsubtab[indpartmax ^ 1];
+  fldthrdtab[1].inddomnptr  = &domnsubtab[indpartmax ^ 1];
   fldthrdtab[1].indvertnbr  = actgrafptr->s.vertlocnbr - indvertlocmax;
   fldthrdtab[1].indpartval  = indpartmax ^ 1;
   fldthrdtab[1].indparttax  = actgrafptr->partgsttax;
   fldthrdtab[1].fldgrafptr  = fldgrafptr;
   fldthrdtab[1].fldpartval  = 1;
-  fldthrdtab[1].fldprocnbr  = indconttab[indpartmax ^ 1] * (actgrafptr->s.procglbnbr - fldprocnbr0); /* Stop if domain limited to one vertex */
+  fldthrdtab[1].fldprocnbr  = indflagtab[indpartmax ^ 1] & (actgrafptr->s.procglbnbr - fldprocnbr0); /* Stop if domain limited to one vertex */
 
   if (actgrafptr->s.proclocnum < fldprocnbr0) {   /* Compute color and rank in our future subpart */
     fldpartval = 0;
     fldprocnum = actgrafptr->s.proclocnum;
-    fldprocnbr = fldprocnbr0;
   }
   else {
     fldpartval = 1;
     fldprocnum = actgrafptr->s.proclocnum - fldprocnbr0;
-    fldprocnbr = actgrafptr->s.procglbnbr - fldprocnbr0;
   }
 
   fldgrafptr->domnorg = *fldthrdtab[fldpartval].inddomnptr; /* Set data of our folded graph */
@@ -298,7 +295,7 @@ KdgraphMapRbPartGraph * restrict const  fldgrafptr)
   fldthrdtab[fldpartval ^ 1].fldproccomm = MPI_COMM_NULL;
 
 #ifdef SCOTCH_PTHREAD
-  if ((indconttab[0] + indconttab[1]) == 2) {     /* If both subjobs have meaningful things to do in parallel     */
+  if ((indflagtab[0] & indflagtab[1]) != 0) {     /* If both subjobs have meaningful things to do in parallel     */
     orggrafdat = actgrafptr->s;                   /* Create a separate graph structure to change its communicator */
     orggrafdat.flagval = (orggrafdat.flagval & ~DGRAPHFREEALL) | DGRAPHFREECOMM;
     fldthrdtab[1].orggrafptr = &orggrafdat;
@@ -338,7 +335,7 @@ kdgraphMapRbPart2 (
 KdgraphMapRbPartGraph * restrict const      grafptr,
 const KdgraphMapRbPartData * restrict const dataptr)
 {
-  ArchDom               domsubtab[2];             /* Temporary subdomains        */
+  ArchDom               domnsubtab[2];            /* Temporary subdomains        */
   Bdgraph               actgrafdat;               /* Active bipartitioning graph */
   KdgraphMapRbPartGraph indgrafdat;               /* Induced folded graph area   */
   Gnum                  comploadavg;
@@ -350,7 +347,7 @@ const KdgraphMapRbPartData * restrict const dataptr)
   o = ((archVar (&mappptr->archdat)) &&           /* If architecture is variable-sized      */
        (grafptr->data.dgrfdat.vertglbnbr <= 1))   /* And source subgraph is of minimal size */
       ? 1                                         /* Then do not bipartition target more    */
-      : archDomBipart (&mappptr->archdat, &grafptr->domnorg, &domsubtab[0], &domsubtab[1]);
+      : archDomBipart (&mappptr->archdat, &grafptr->domnorg, &domnsubtab[0], &domnsubtab[1]);
 
   switch (o) {
     case 1 :                                      /* If target domain is terminal */
@@ -365,23 +362,23 @@ const KdgraphMapRbPartData * restrict const dataptr)
     return     (1);
   }
   
-  o = bdgraphInit (&actgrafdat, &grafptr->data.dgrfdat, NULL, &mappptr->archdat, domsubtab); /* Create active graph */
-  actgrafdat.levlnum = grafptr->levlnum;          /* Initial level of bipartition graph is DRB recursion level      */
+  o = bdgraphInit (&actgrafdat, &grafptr->data.dgrfdat, NULL, &mappptr->archdat, domnsubtab); /* Create active graph */
+  actgrafdat.levlnum = grafptr->levlnum;          /* Initial level of bipartition graph is DRB recursion level       */
 
   comploadavg = (double) actgrafdat.s.veloglbsum / (double) archDomWght (&mappptr->archdat, &grafptr->domnorg);
   actgrafdat.compglbload0min = actgrafdat.compglbload0avg -
-                               (Gnum) MIN ((dataptr->comploadmax - comploadavg) * actgrafdat.domwght[0],
-                                           (comploadavg - dataptr->comploadmin) * actgrafdat.domwght[1]);
+                               (Gnum) MIN ((dataptr->comploadmax - comploadavg) * actgrafdat.domnwght[0],
+                                           (comploadavg - dataptr->comploadmin) * actgrafdat.domnwght[1]);
   actgrafdat.compglbload0max = actgrafdat.compglbload0avg +
-                               (Gnum) MIN ((comploadavg - dataptr->comploadmin) * actgrafdat.domwght[0],
-                                           (dataptr->comploadmax - comploadavg) * actgrafdat.domwght[1]);
+                               (Gnum) MIN ((comploadavg - dataptr->comploadmin) * actgrafdat.domnwght[0],
+                                           (dataptr->comploadmax - comploadavg) * actgrafdat.domnwght[1]);
 
   if ((o != 0) || (bdgraphBipartSt (&actgrafdat, dataptr->paraptr->stratsep) != 0)) { /* Bipartition edge-separation graph   */
     bdgraphExit (&actgrafdat);
     return      (1);
   }
 
-  o = kdgraphMapRbPartFold (&actgrafdat, mappptr, domsubtab, &indgrafdat);
+  o = kdgraphMapRbPartFold (&actgrafdat, mappptr, domnsubtab, &indgrafdat);
 
   bdgraphExit (&actgrafdat);                      /* Free additional bipartitioning data   */
   dgraphExit  (&grafptr->data.dgrfdat);           /* Free graph before going to next level */
