@@ -1,4 +1,4 @@
-/* Copyright 2004,2007,2009-2011 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007,2009-2016,2018 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -34,6 +34,7 @@
 /**   NAME       : library_arch.c                          **/
 /**                                                        **/
 /**   AUTHOR     : Francois PELLEGRINI                     **/
+/**                Sebastien FOURESTIER (v6.0)             **/
 /**                                                        **/
 /**   FUNCTION   : This module is the API for the target   **/
 /**                architecture handling routines of the   **/
@@ -51,6 +52,8 @@
 /**                                 to   : 12 sep 2007     **/
 /**                # Version 5.1  : from : 05 jun 2009     **/
 /**                                 to   : 13 feb 2011     **/
+/**                # Version 6.0  : from : 14 feb 2011     **/
+/**                                 to     05 jun 2018     **/
 /**                                                        **/
 /************************************************************/
 
@@ -68,6 +71,7 @@
 #include "arch_cmpltw.h"
 #include "arch_hcub.h"
 #include "arch_mesh.h"
+#include "arch_sub.h"
 #include "arch_tleaf.h"
 #include "arch_torus.h"
 #include "arch_vcmplt.h"
@@ -108,11 +112,11 @@ SCOTCH_archInit (
 SCOTCH_Arch * const         archptr)
 {
   if (sizeof (SCOTCH_Num) != sizeof (Anum)) {
-    errorPrint ("SCOTCH_archInit: internal error (1)");
+    errorPrint (STRINGIFY (SCOTCH_archInit) ": internal error (1)");
     return     (1);
   }
   if (sizeof (SCOTCH_Arch) < sizeof (Arch)) {
-    errorPrint ("SCOTCH_archInit: internal error (2)");
+    errorPrint (STRINGIFY (SCOTCH_archInit) ": internal error (2)");
     return     (1);
   }
 
@@ -219,21 +223,22 @@ const SCOTCH_Arch * const   archptr)
 int
 SCOTCH_archCmplt (
 SCOTCH_Arch * const         archptr,
-const SCOTCH_Num            numnbr)
+const SCOTCH_Num            termnbr)
 {
   Arch *              tgtarchptr;
   ArchCmplt *         tgtarchdatptr;
 
   if (sizeof (SCOTCH_Num) != sizeof (Gnum)) {
-    errorPrint ("SCOTCH_archCmplt: internal error");
+    errorPrint (STRINGIFY (SCOTCH_archCmplt) ": internal error");
     return     (1);
   }
 
   tgtarchptr    = (Arch *) archptr;
   tgtarchdatptr = (ArchCmplt *) (void *) (&tgtarchptr->data);
 
-  tgtarchptr->class     = archClass ("cmplt");
-  tgtarchdatptr->numnbr = (Anum) numnbr;
+  tgtarchptr->class      = archClass ("cmplt");
+  tgtarchptr->flagval    = tgtarchptr->class->flagval; /* Copy architecture flag */
+  tgtarchdatptr->termnbr = (Anum) termnbr;
 
   return (0);
 }
@@ -251,12 +256,13 @@ const SCOTCH_Num * const    velotab)
   Arch *              tgtarchptr;
 
   if (sizeof (SCOTCH_Num) != sizeof (Gnum)) {
-    errorPrint ("SCOTCH_archCmpltw: internal error");
+    errorPrint (STRINGIFY (SCOTCH_archCmpltw) ": internal error");
     return     (1);
   }
 
-  tgtarchptr        = (Arch *) archptr;
-  tgtarchptr->class = archClass ("cmpltw");
+  tgtarchptr          = (Arch *) archptr;
+  tgtarchptr->class   = archClass ("cmpltw");
+  tgtarchptr->flagval = tgtarchptr->class->flagval; /* Copy architecture flag */
 
   return (archCmpltwArchBuild ((ArchCmpltw *) (void *) (&tgtarchptr->data), vertnbr, velotab));
 }
@@ -268,21 +274,22 @@ const SCOTCH_Num * const    velotab)
 int
 SCOTCH_archHcub (
 SCOTCH_Arch * const         archptr,
-const SCOTCH_Num            dimmax)               /*+ Number of dimensions +*/
+const SCOTCH_Num            dimnnbr)              /*+ Number of dimensions +*/
 {
   Arch *              tgtarchptr;
   ArchHcub *          tgtarchdatptr;
 
   if (sizeof (SCOTCH_Num) != sizeof (Gnum)) {
-    errorPrint ("SCOTCH_archHcub: internal error");
+    errorPrint (STRINGIFY (SCOTCH_archHcub) ": internal error");
     return     (1);
   }
 
   tgtarchptr    = (Arch *) archptr;
   tgtarchdatptr = (ArchHcub *) (void *) (&tgtarchptr->data);
 
-  tgtarchptr->class     = archClass ("hcub");
-  tgtarchdatptr->dimmax = (Anum) dimmax;
+  tgtarchptr->class      = archClass ("hcub");
+  tgtarchptr->flagval    = tgtarchptr->class->flagval; /* Copy architecture flag */
+  tgtarchdatptr->dimnnbr = (Anum) dimnnbr;
 
   return (0);
 }
@@ -301,7 +308,7 @@ const SCOTCH_Num            dimyval)
   ArchMesh2 *         tgtarchdatptr;
 
   if (sizeof (SCOTCH_Num) != sizeof (Gnum)) {
-    errorPrint ("SCOTCH_archMesh2: internal error");
+    errorPrint (STRINGIFY (SCOTCH_archMesh2) ": internal error");
     return     (1);
   }
 
@@ -309,8 +316,10 @@ const SCOTCH_Num            dimyval)
   tgtarchdatptr = (ArchMesh2 *) (void *) (&tgtarchptr->data);
 
   tgtarchptr->class   = archClass ("mesh2D");
-  tgtarchdatptr->c[0] = (Anum) dimxval;
-  tgtarchdatptr->c[1] = (Anum) dimyval;
+  tgtarchptr->flagval = tgtarchptr->class->flagval; /* Copy architecture flag */
+  tgtarchdatptr->dimnnbr = 2;
+  tgtarchdatptr->c[0]    = (Anum) dimxval;
+  tgtarchdatptr->c[1]    = (Anum) dimyval;
 
   return (0);
 }
@@ -330,7 +339,7 @@ const SCOTCH_Num            dimzval)
   ArchMesh3 *         tgtarchdatptr;
 
   if (sizeof (SCOTCH_Num) != sizeof (Gnum)) {
-    errorPrint ("SCOTCH_archMesh3: internal error");
+    errorPrint (STRINGIFY (SCOTCH_archMesh3) ": internal error");
     return     (1);
   }
 
@@ -338,9 +347,11 @@ const SCOTCH_Num            dimzval)
   tgtarchdatptr = (ArchMesh3 *) (void *) (&tgtarchptr->data);
 
   tgtarchptr->class   = archClass ("mesh3D");
-  tgtarchdatptr->c[0] = (Anum) dimxval;
-  tgtarchdatptr->c[1] = (Anum) dimyval;
-  tgtarchdatptr->c[2] = (Anum) dimzval;
+  tgtarchptr->flagval = tgtarchptr->class->flagval; /* Copy architecture flag */
+  tgtarchdatptr->dimnnbr = 3;
+  tgtarchdatptr->c[0]    = (Anum) dimxval;
+  tgtarchdatptr->c[1]    = (Anum) dimyval;
+  tgtarchdatptr->c[2]    = (Anum) dimzval;
 
   return (0);
 }
@@ -349,6 +360,67 @@ const SCOTCH_Num            dimzval)
 **
 */
 
+int
+SCOTCH_archMeshX (
+SCOTCH_Arch * const         archptr,
+const SCOTCH_Num            dimnnbr,              /*+ Number of dimensions in architecture +*/
+const SCOTCH_Num * const    dimntab)              /*+ Array of dimensions                  +*/
+{
+  Arch *              tgtarchptr;
+  ArchMeshX *         tgtarchdatptr;
+
+  if (sizeof (SCOTCH_Num) != sizeof (Gnum)) {
+    errorPrint (STRINGIFY (SCOTCH_archMeshX) ": internal error");
+    return     (1);
+  }
+  if (dimnnbr > ARCHMESHDIMNMAX) {
+    errorPrint (STRINGIFY (SCOTCH_archMeshX) ": too many dimensions");
+    return     (1);
+  }
+
+  tgtarchptr    = (Arch *) archptr;
+  tgtarchdatptr = (ArchMeshX *) (void *) (&tgtarchptr->data);
+
+  tgtarchptr->class      = archClass ("meshXD");
+  tgtarchptr->flagval    = tgtarchptr->class->flagval; /* Copy architecture flag */
+  tgtarchdatptr->dimnnbr = dimnnbr;
+  memCpy (tgtarchdatptr->c, dimntab, dimnnbr * sizeof (SCOTCH_Num)); /* Copy dimension array */
+
+  return (0);
+}
+
+/*
+**
+*/
+
+int
+SCOTCH_archSub (
+SCOTCH_Arch * const         subarchptr,
+SCOTCH_Arch * const         orgarchptr,
+const SCOTCH_Num            vnumnbr,              /*+ Number of dimensions +*/
+const SCOTCH_Num * const    vnumtab)
+{
+  Arch *              tgtarchptr;
+  ArchSub *           tgtarchdatptr;
+
+  if (sizeof (SCOTCH_Num) != sizeof (Gnum)) {
+    errorPrint (STRINGIFY (SCOTCH_archSub) ": internal error");
+    return     (2);
+  }
+
+  tgtarchptr    = (Arch *) subarchptr;
+  tgtarchdatptr = (ArchSub *) (void *) (&tgtarchptr->data);
+
+  tgtarchptr->class   = archClass ("sub");
+  tgtarchptr->flagval = tgtarchptr->class->flagval; /* Copy architecture flag */
+
+  return (archSubArchBuild (tgtarchdatptr, (Arch *) orgarchptr, (Gnum) vnumnbr, (Gnum *) vnumtab));
+}
+
+/*
+**
+*/
+
 int
 SCOTCH_archTleaf (
 SCOTCH_Arch * const         archptr,
@@ -362,28 +434,68 @@ const SCOTCH_Num * const    linktab)              /*+ Link cost array, by increa
   ArchTleaf *         tgtarchdatptr;
 
   if (sizeof (SCOTCH_Num) != sizeof (Gnum)) {
-    errorPrint ("SCOTCH_archTleaf: internal error");
+    errorPrint (STRINGIFY (SCOTCH_archTleaf) ": internal error");
     return     (1);
   }
 
-  tgtarchptr        = (Arch *) archptr;
-  tgtarchdatptr     = (ArchTleaf *) (void *) (&tgtarchptr->data);
-  tgtarchptr->class = archClass ("tleaf");
+  tgtarchptr          = (Arch *) archptr;
+  tgtarchdatptr       = (ArchTleaf *) (void *) (&tgtarchptr->data);
+  tgtarchptr->class   = archClass ("tleaf");
+  tgtarchptr->flagval = tgtarchptr->class->flagval; /* Copy architecture flag */
 
   if ((tgtarchdatptr->sizetab = memAlloc ((levlnbr * 2 + 1) * sizeof (Anum))) == NULL) { /* TRICK: One more slot for linktab[-1] */
-    errorPrint ("SCOTCH_archTleaf: out of memory");
+    errorPrint (STRINGIFY (SCOTCH_archTleaf) ": out of memory");
     return     (1);
   }
   tgtarchdatptr->levlnbr     = (Anum) levlnbr;
   tgtarchdatptr->linktab     = tgtarchdatptr->sizetab + tgtarchdatptr->levlnbr + 1;
   tgtarchdatptr->linktab[-1] = 0;                 /* TRICK: Dummy slot for for level-0 communication */
+  tgtarchdatptr->permtab     = NULL;              /* Assume no permutation array                     */
 
   for (levlnum = 0, sizeval = 1; levlnum < tgtarchdatptr->levlnbr; levlnum ++) {
     tgtarchdatptr->sizetab[levlnum] = sizetab[levlnum];
     tgtarchdatptr->linktab[levlnum] = linktab[levlnum];
     sizeval *= tgtarchdatptr->sizetab[levlnum];
   }
-  tgtarchdatptr->sizeval = sizeval;
+  tgtarchdatptr->termnbr = sizeval;
+
+  return (0);
+}
+
+/*
+**
+*/
+
+int
+SCOTCH_archLtleaf (
+SCOTCH_Arch * const         archptr,
+const SCOTCH_Num            levlnbr,              /*+ Number of levels in architecture            +*/
+const SCOTCH_Num * const    sizetab,              /*+ Size array, by increasing level number      +*/
+const SCOTCH_Num * const    linktab,              /*+ Link cost array, by increasing level number +*/
+const SCOTCH_Num            permnbr,              /*+ Number of permutation indices               +*/
+const SCOTCH_Num * const    permtab)              /*+ Permutation array                           +*/
+{
+  Anum                permnum;
+  Arch *              tgtarchptr;
+  ArchTleaf *         tgtarchdatptr;
+
+  if (SCOTCH_archTleaf (archptr, levlnbr, sizetab, linktab) != 0) /* Build tree part */
+    return (1);
+  tgtarchptr        = (Arch *) archptr;
+  tgtarchdatptr     = (ArchTleaf *) (void *) (&tgtarchptr->data);
+  tgtarchptr->class = archClass ("ltleaf");       /* Override class */
+
+  if ((tgtarchdatptr->permtab = memAlloc (permnbr * 2 * sizeof (Anum))) == NULL) { /* TRICK: space for peritab too */
+    errorPrint (STRINGIFY (SCOTCH_archLtleaf) ": out of memory");
+    return     (1);
+  }
+  tgtarchdatptr->permnbr = (Anum) permnbr;
+  tgtarchdatptr->peritab = tgtarchdatptr->permtab + permnbr;
+
+  for (permnum = 0; permnum < tgtarchdatptr->permnbr; permnum ++)
+    tgtarchdatptr->permtab[permnum] = permtab[permnum];
+  for (permnum = 0; permnum < tgtarchdatptr->permnbr; permnum ++) /* Build inverse permutation */
+    tgtarchdatptr->peritab[tgtarchdatptr->permtab[permnum]] = permnum;
 
   return (0);
 }
@@ -399,19 +511,21 @@ const SCOTCH_Num            dimxval,
 const SCOTCH_Num            dimyval)
 {
   Arch *              tgtarchptr;
-  ArchTorus2 *        tgtarchdatptr;
+  ArchTorusX *        tgtarchdatptr;
 
   if (sizeof (SCOTCH_Num) != sizeof (Gnum)) {
-    errorPrint ("SCOTCH_archTorus2: internal error");
+    errorPrint (STRINGIFY (SCOTCH_archTorus2) ": internal error");
     return     (1);
   }
 
   tgtarchptr    = (Arch *) archptr;
-  tgtarchdatptr = (ArchTorus2 *) (void *) (&tgtarchptr->data);
+  tgtarchdatptr = (ArchTorusX *) (void *) (&tgtarchptr->data);
 
-  tgtarchptr->class   = archClass ("torus2D");
-  tgtarchdatptr->c[0] = (Anum) dimxval;
-  tgtarchdatptr->c[1] = (Anum) dimyval;
+  tgtarchptr->class      = archClass ("torus2D");
+  tgtarchptr->flagval    = tgtarchptr->class->flagval; /* Copy architecture flag */
+  tgtarchdatptr->dimnnbr = 2;
+  tgtarchdatptr->c[0]    = (Anum) dimxval;
+  tgtarchdatptr->c[1]    = (Anum) dimyval;
 
   return (0);
 }
@@ -428,20 +542,55 @@ const SCOTCH_Num            dimyval,
 const SCOTCH_Num            dimzval)
 {
   Arch *              tgtarchptr;
-  ArchTorus3 *        tgtarchdatptr;
+  ArchTorusX *        tgtarchdatptr;
 
   if (sizeof (SCOTCH_Num) != sizeof (Gnum)) {
-    errorPrint ("SCOTCH_archTorus3: internal error");
+    errorPrint (STRINGIFY (SCOTCH_archTorus3) ": internal error");
     return     (1);
   }
 
   tgtarchptr    = (Arch *) archptr;
-  tgtarchdatptr = (ArchTorus3 *) (void *) (&tgtarchptr->data);
+  tgtarchdatptr = (ArchTorusX *) (void *) (&tgtarchptr->data);
 
-  tgtarchptr->class   = archClass ("torus3D");
-  tgtarchdatptr->c[0] = (Anum) dimxval;
-  tgtarchdatptr->c[1] = (Anum) dimyval;
-  tgtarchdatptr->c[2] = (Anum) dimzval;
+  tgtarchptr->class      = archClass ("torus3D");
+  tgtarchptr->flagval    = tgtarchptr->class->flagval; /* Copy architecture flag */
+  tgtarchdatptr->dimnnbr = 3;
+  tgtarchdatptr->c[0]    = (Anum) dimxval;
+  tgtarchdatptr->c[1]    = (Anum) dimyval;
+  tgtarchdatptr->c[2]    = (Anum) dimzval;
+
+  return (0);
+}
+
+/*
+**
+*/
+
+int
+SCOTCH_archTorusX (
+SCOTCH_Arch * const         archptr,
+const SCOTCH_Num            dimnnbr,              /*+ Number of dimensions in architecture +*/
+const SCOTCH_Num * const    dimntab)              /*+ Array of dimensions                  +*/
+{
+  Arch *              tgtarchptr;
+  ArchTorusX *        tgtarchdatptr;
+
+  if (sizeof (SCOTCH_Num) != sizeof (Gnum)) {
+    errorPrint (STRINGIFY (SCOTCH_archTorusX) ": internal error");
+    return     (1);
+  }
+  if (dimnnbr > ARCHMESHDIMNMAX) {
+    errorPrint (STRINGIFY (SCOTCH_archTorusX) ": too many dimensions");
+    return     (1);
+  }
+
+  tgtarchptr    = (Arch *) archptr;
+  tgtarchdatptr = (ArchTorusX *) (void *) (&tgtarchptr->data);
+
+  tgtarchptr->class      = archClass ("torusXD");
+  tgtarchptr->flagval    = tgtarchptr->class->flagval; /* Copy architecture flag */
+  tgtarchdatptr->dimnnbr = dimnnbr;
+  memCpy (tgtarchdatptr->c, dimntab, dimnnbr * sizeof (SCOTCH_Num)); /* Copy dimension array */
 
   return (0);
 }
@@ -457,13 +606,14 @@ SCOTCH_Arch * const         archptr)
   Arch *              tgtarchptr;
 
   if (sizeof (SCOTCH_Num) != sizeof (Gnum)) {
-    errorPrint ("SCOTCH_archVcmplt: internal error");
+    errorPrint (STRINGIFY (SCOTCH_archVcmplt) ": internal error");
     return     (1);
   }
 
   tgtarchptr = (Arch *) archptr;
 
-  tgtarchptr->class = archClass ("varcmplt");
+  tgtarchptr->class   = archClass ("varcmplt");
+  tgtarchptr->flagval = tgtarchptr->class->flagval; /* Copy architecture flag */
 
   return (0);
 }
@@ -479,13 +629,14 @@ SCOTCH_Arch * const         archptr)
   Arch *              tgtarchptr;
 
   if (sizeof (SCOTCH_Num) != sizeof (Gnum)) {
-    errorPrint ("SCOTCH_archVhcub: internal error");
+    errorPrint (STRINGIFY (SCOTCH_archVhcub) ": internal error");
     return     (1);
   }
 
   tgtarchptr = (Arch *) archptr;
 
-  tgtarchptr->class = archClass ("vhcub");
+  tgtarchptr->class   = archClass ("varhcub");
+  tgtarchptr->flagval = tgtarchptr->class->flagval; /* Copy architecture flag */
 
   return (0);
 }
