@@ -1,4 +1,4 @@
-/* Copyright 2007-2009 ENSEIRB, INRIA & CNRS
+/* Copyright 2007-2009,2014 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -44,6 +44,8 @@
 /**                                 to     01 aug 2007     **/
 /**                # Version 5.1  : from : 05 nov 2007     **/
 /**                                 to     26 may 2009     **/
+/**                # Version 6.0  : from : 01 may 2014     **/
+/**                                 to     30 sep 2014     **/
 /**                                                        **/
 /************************************************************/
 
@@ -88,7 +90,7 @@ static union {
 static union {
   VdgraphSeparateMlParam    param;
   StratNodeMethodData       padding;
-} vdgraphseparatedefaultml = { { 5, 1000, 100, 0, 0.8L, &stratdummy, &stratdummy, &stratdummy, 1 } };
+} vdgraphseparatedefaultml = { { 5, 1000, 2, 10000, 0.8L, &stratdummy, &stratdummy, &stratdummy } };
 
 static union {
   VdgraphSeparateSqParam    param;
@@ -150,26 +152,24 @@ static StratParamTab        vdgraphseparatestparatab[] = { /* Distributed graph
                                 (byte *) &vdgraphseparatedefaultml.param,
                                 (byte *) &vdgraphseparatedefaultml.param.passnbr,
                                 NULL },
-                              { VDGRAPHSEPASTMETHML,  STRATPARAMINT,    "proc",
-                                (byte *) &vdgraphseparatedefaultml.param,
-                                (byte *) &vdgraphseparatedefaultml.param.seqnbr,
-                                NULL },
                               { VDGRAPHSEPASTMETHML,  STRATPARAMINT,    "vert",
                                 (byte *) &vdgraphseparatedefaultml.param,
                                 (byte *) &vdgraphseparatedefaultml.param.coarnbr,
                                 NULL },
                               { VDGRAPHSEPASTMETHML,  STRATPARAMINT,    "dvert",
                                 (byte *) &vdgraphseparatedefaultml.param,
-                                (byte *) &vdgraphseparatedefaultml.param.dupmax,
+                                (byte *) &vdgraphseparatedefaultml.param.foldmax,
                                 NULL },
-                              { VDGRAPHSEPASTMETHML,  STRATPARAMINT,    "dlevl",
+                              { VDGRAPHSEPASTMETHML,  STRATPARAMCASE,   "fold",
                                 (byte *) &vdgraphseparatedefaultml.param,
-                                (byte *) &vdgraphseparatedefaultml.param.duplvlmax,
-                                NULL },
+                                (byte *) &vdgraphseparatedefaultml.param.foldval,
+                                (void *) "nfd" },
                               { VDGRAPHSEPASTMETHML,  STRATPARAMDOUBLE, "rat",
                                 (byte *) &vdgraphseparatedefaultml.param,
                                 (byte *) &vdgraphseparatedefaultml.param.coarrat,
                                 NULL },
+                              { VDGRAPHSEPASTMETHML,  STRATPARAMDEPRECATED | STRATPARAMINT, "dlevl", NULL, NULL, NULL }, /* Wait until MUMPS 5.0 */
+                              { VDGRAPHSEPASTMETHML,  STRATPARAMDEPRECATED | STRATPARAMINT, "proc",  NULL, NULL, NULL },
                               { VDGRAPHSEPASTMETHSQ,  STRATPARAMSTRAT,  "strat",
                                 (byte *) &vdgraphseparatedefaultsq.param,
                                 (byte *) &vdgraphseparatedefaultsq.param.strat,
@@ -236,6 +236,7 @@ const Strat * restrict const  strat)              /*+ Separation strategy
 {
   StratTest           val;
   VdgraphStore        savetab[2];                 /* Results of the two strategies */
+  Gnum                compglbload2;               /* Saved global separator load   */
   int                 o;
 #ifdef SCOTCH_DEBUG_VDGRAPH2
   MPI_Comm            proccommold;                /* Save area for old communicator */
@@ -307,8 +308,9 @@ const Strat * restrict const  strat)              /*+ Separation strategy
       if (vdgraphSeparateSt (grafptr, strat->data.select.strat[1]) != 0) /* If second strategy didn't work */
         vdgraphStoreUpdt (grafptr, &savetab[1]);  /* Restore initial bipartition as its result             */
 
-      if ( (savetab[0].fronglbnbr <  grafptr->compglbsize[2]) || /* If first strategy is better */
-          ((savetab[0].fronglbnbr == grafptr->compglbsize[2]) &&
+      compglbload2 = grafptr->s.veloglbsum - savetab[0].compglbload[0] - savetab[0].compglbload[1]; /* Compute saved separator load */
+      if ( (compglbload2 <  grafptr->compglbload[2]) || /* If first strategy is better */
+          ((compglbload2 == grafptr->compglbload[2]) &&
            (abs (savetab[0].compglbloaddlt) < abs (grafptr->compglbloaddlt))))
         vdgraphStoreUpdt (grafptr, &savetab[0]);  /* Restore its result */
 
