@@ -1,4 +1,4 @@
-/* Copyright 2004,2007 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007,2008 ENSEIRB, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -43,6 +43,8 @@
 /**                                 to     18 aug 2004     **/
 /**                # Version 5.0  : from : 12 sep 2007     **/
 /**                                 to     24 mar 2008     **/
+/**                # Version 5.1  : from : 09 nov 2008     **/
+/**                                 to     09 nov 2008     **/
 /**                                                        **/
 /************************************************************/
 
@@ -407,7 +409,7 @@ const VmeshSeparateGgParam * restrict const paraptr) /*+ Method parameters    +*
   }
   meshptr->ncmpsize[0] = meshptr->m.vnodnbr - (ncmpsize1 + ncmpsize2);
   meshptr->ncmpsize[1] = ncmpsize1;
-  meshptr->ncmpsize[2] = ncmpsize2;
+  meshptr->fronnbr     = ncmpsize2;
 
 #ifdef SCOTCH_DEBUG_VMESH2
   if (vmeshCheck (meshptr) != 0) {
@@ -421,7 +423,7 @@ const VmeshSeparateGgParam * restrict const paraptr) /*+ Method parameters    +*
   memFree      (vexxtab);                         /* Free group leader */
   gainTablExit (tablptr);
 
-printf ("GG Sepa\tsize=%ld\tload=%ld\tbal=%ld\n", (long) meshptr->fronnbr, (long) meshptr->ncmpload[2], (long) meshptr->ncmploaddlt);
+/* printf ("GG Sepa\tsize=%ld\tload=%ld\tbal=%ld\n", (long) meshptr->fronnbr, (long) meshptr->ncmpload[2], (long) meshptr->ncmploaddlt); */
 
   return (0);
 }
