@@ -1,4 +1,4 @@
-/* Copyright 2004,2007,2008,2010-2012 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2004,2007,2008,2010-2012,2014,2018 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -57,7 +57,7 @@
 /**                # Version 5.1  : from : 01 jul 2010     **/
 /**                                 to   : 14 feb 2011     **/
 /**                # Version 6.0  : from : 01 jan 2012     **/
-/**                                 to   : 01 jan 2012     **/
+/**                                 to   : 10 jul 2018     **/
 /**                                                        **/
 /************************************************************/
 
@@ -72,14 +72,18 @@
 #include "scotch.h"
 #include "gmk_ub2.h"
 
-#define ngbadd(v)                 if ((v) != vertnum) {           \
-                                    int                   k;      \
-                                    for (k = 0; k < ngbnbr; k ++) \
-                                      if ((v) == ngbtab[k])       \
-                                        break;                    \
-                                    if (k == ngbnbr)              \
-                                      ngbtab[ngbnbr ++] = (v);    \
-                                  }
+#define ngbadd(v)                 do {                              \
+                                    int                 t;          \
+                                    t = (v);                        \
+                                    if (t != vertnum) {             \
+                                      int                 k;        \
+                                      for (k = 0; k < ngbnbr; k ++) \
+                                        if (t == ngbtab[k])         \
+                                          break;                    \
+                                      if (k == ngbnbr)              \
+                                        ngbtab[ngbnbr ++] = t;      \
+                                    }                               \
+                                  } while (0);
 
 /*
 **  The static definitions.
@@ -88,7 +92,7 @@
 static int                  C_paraNum = 0;        /* Number of parameters       */
 static int                  C_fileNum = 0;        /* Number of file in arg list */
 static File                 C_fileTab[C_FILENBR] = { /* The file array          */
-                              { "-", NULL, "w" } };
+                              { FILEMODEW } };
 
 static const char *         C_usageList[] = {
   "gmk_ub2 <dim> [<output source file>] <options>",
@@ -123,8 +127,8 @@ char *                      argv[])
     return     (0);
   }
 
-  for (i = 0; i < C_FILENBR; i ++)                /* Set default stream pointers */
-    C_fileTab[i].pntr = (C_fileTab[i].mode[0] == 'r') ? stdin : stdout;
+  fileBlockInit (C_fileTab, C_FILENBR);           /* Set default stream pointers */
+
   for (i = 1; i < argc; i ++) {                   /* Loop for all option codes                        */
     if ((argv[i][0] != '-') || (argv[i][1] == '\0') || (argv[i][1] == '.')) { /* If found a file name */
       if (C_paraNum < 1) {                        /* If number of parameters not reached              */
@@ -136,7 +140,7 @@ char *                      argv[])
         continue;                                 /* Process the other parameters */
       }
       if (C_fileNum < C_FILEARGNBR)               /* A file name has been given */
-        C_fileTab[C_fileNum ++].name = argv[i];
+        fileBlockName (C_fileTab, C_fileNum ++) = argv[i];
       else {
         errorPrint ("main: too many file names given");
         return     (1);
@@ -150,7 +154,7 @@ char *                      argv[])
           return     (0);
         case 'V' :
           fprintf (stderr, "gmk_ub2, version " SCOTCH_VERSION_STRING "\n");
-          fprintf (stderr, "Copyright 2004,2007,2008,2010-2012 IPB, Universite de Bordeaux, INRIA & CNRS, France\n");
+          fprintf (stderr, "Copyright 2004,2007,2008,2010-2012,2014,2018 IPB, Universite de Bordeaux, INRIA & CNRS, France\n");
           fprintf (stderr, "This software is libre/free software under CeCILL-C -- see the user's manual for more information\n");
           return  (0);
         default :
@@ -185,8 +189,5 @@ char *                      argv[])
 
   fileBlockClose (C_fileTab, C_FILENBR);          /* Always close explicitely to end eventual (un)compression tasks */
 
-#ifdef COMMON_PTHREAD
-  pthread_exit ((void *) 0);                      /* Allow potential (un)compression tasks to complete */
-#endif /* COMMON_PTHREAD */
   return (0);
 }
