@@ -1,4 +1,4 @@
-/* Copyright 2004,2007,2008,2010,2011 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007,2008,2010-2012,2014,2018 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -45,6 +45,8 @@
 /**                                 to   : 16 mar 2008     **/
 /**                # Version 5.1  : from : 20 apr 2010     **/
 /**                                 to   : 14 feb 2011     **/
+/**                # Version 6.0  : from : 01 jan 2012     **/
+/**                                 to   : 10 jul 2018     **/
 /**                                                        **/
 /**   NOTES      : # The cost analysis routine leaves the  **/
 /**                  memory management to malloc and free  **/
@@ -71,9 +73,9 @@
 
 static int                  C_fileNum    = 0;     /* Number of file in arg list  */
 static File                 C_fileTab[3] = {      /* File array                  */
-                              { "-", NULL, "r" },
-                              { "-", NULL, "r" },
-                              { "-", NULL, "w" } };
+                              { FILEMODER },
+                              { FILEMODER },
+                              { FILEMODEW } };
 
 static const char *         C_usageList[] = {
   "gotst [<input graph file> [<input ordering file> [<output data file>]]] <options>",
@@ -111,12 +113,12 @@ char *                      argv[])
     return     (0);
   }
 
-  for (i = 0; i < C_FILENBR; i ++)                /* Set default stream pointers */
-    C_fileTab[i].pntr = (C_fileTab[i].mode[0] == 'r') ? stdin : stdout;
+  fileBlockInit (C_fileTab, C_FILENBR);           /* Set default stream pointers */
+
   for (i = 1; i < argc; i ++) {                   /* Loop for all option codes                        */
     if ((argv[i][0] != '-') || (argv[i][1] == '\0') || (argv[i][1] == '.')) { /* If found a file name */
       if (C_fileNum < C_FILEARGNBR)               /* File name has been given                         */
-        C_fileTab[C_fileNum ++].name = argv[i];
+        fileBlockName (C_fileTab, C_fileNum ++) = argv[i];
       else {
         errorPrint ("main: too many file names given");
         return     (1);
@@ -130,7 +132,7 @@ char *                      argv[])
           return     (0);
         case 'V' :
           fprintf (stderr, "gotst, version " SCOTCH_VERSION_STRING "\n");
-          fprintf (stderr, "Copyright 2004,2007,2008,2010,2011 ENSEIRB, INRIA & CNRS, France\n");
+          fprintf (stderr, "Copyright 2004,2007,2008,2010-2012,2014,2018 IPB, Universite de Bordeaux, INRIA & CNRS, France\n");
           fprintf (stderr, "This software is libre/free software under CeCILL-C -- see the user's manual for more information\n");
           return  (0);
         default :
@@ -172,9 +174,6 @@ char *                      argv[])
   SCOTCH_graphOrderExit (&grafdat, &ordedat);
   SCOTCH_graphExit      (&grafdat);
 
-#ifdef COMMON_PTHREAD
-  pthread_exit ((void *) 0);                      /* Allow potential (un)compression tasks to complete */
-#endif /* COMMON_PTHREAD */
   return (0);
 }
 
