@@ -1,4 +1,4 @@
-/* Copyright 2004,2007,2008,2010-2012,2014 IPB, Universite de Bordeaux, INRIA & CNRS
+/* Copyright 2004,2007,2008,2010-2012,2014,2018 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -57,7 +57,7 @@
 /**                # Version 5.1  : from : 25 oct 2007     **/
 /**                                 to     14 feb 2011     **/
 /**                # Version 6.0  : from : 16 oct 2010     **/
-/**                                 to     12 nov 2014     **/
+/**                                 to     10 jul 2018     **/
 /**                                                        **/
 /************************************************************/
 
@@ -79,10 +79,10 @@
 
 static int                  C_fileNum = 0;        /* Number of file in arg list */
 File                        C_fileTab[C_FILENBR] = { /* The file array; public  */
-                              { "r" },
-                              { "r" },
-                              { "r" },
-                              { "w" } };
+                              { FILEMODER },
+                              { FILEMODER },
+                              { FILEMODER },
+                              { FILEMODEW } };
 
 static unsigned int         C_geoFlag = C_GEOFLAGDEFAULT; /* Geometry flag */
 
@@ -186,7 +186,7 @@ char *                      argv[])
           break;
         case 'V' :
           fprintf (stderr, "gout, version " SCOTCH_VERSION_STRING "\n");
-          fprintf (stderr, "Copyright 2004,2007,2008,2010-2012,2014 IPB, Universite de Bordeaux, INRIA & CNRS, France\n");
+          fprintf (stderr, "Copyright 2004,2007,2008,2010-2012,2014,2018 IPB, Universite de Bordeaux, INRIA & CNRS, France\n");
           fprintf (stderr, "This software is libre/free software under CeCILL-C -- see the user's manual for more information\n");
           return  (0);
         default :
@@ -218,9 +218,6 @@ char *                      argv[])
   C_geoExit        (&geo);
   SCOTCH_graphExit (&grafdat.grafdat);
 
-#ifdef COMMON_PTHREAD
-  pthread_exit ((void *) 0);                      /* Allow potential (un)compression tasks to complete */
-#endif /* COMMON_PTHREAD */
   return (0);
 }
 
